﻿using System;
using System.IO;
using System.Runtime.CompilerServices;
using System.Text;
using System.Xml;
using System.Linq;
using System.Collections.Generic;
using DataContracts;
using System.Text.RegularExpressions;

namespace Ato.EN.IntegrationServices.CodeGenerationWRE
{
    public partial class WRE2017Validator
    {
        private List<ProcessMessageDocument> ValidateGEN001023(List<ProcessMessageDocument> response, WRE2017 report)
        {
            #region VR.ATO.GEN.001023

            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (RP:entity.identifier.TFN <> ANY OCCURRENCE OF entity.identifier.TFN)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            AnyOccurrence(^TFN2, ^TFN2 <> ^TFN)
    
            Data Elements:
    
            ^TFN = RP
    
            ^TFN2 = Other TFNs
            */

            if (report.RPCarSeqNumContextCollection != null)
            {
                foreach (WRE2017.RPCarSeqNumContext context in report.RPCarSeqNumContextCollection)
                {
                    if (context.IdentifierTFN != report.RPIdentifierTFN)
                    {
                        ProcessMessageDocument processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.001023",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Unexpected TFN mismatch with Reporting Party",
                            LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                            Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                        };
                        response.Add(processMessage);
                    }
                }
            }

            if (report.RPClthngSeqNumContextCollection != null)
            {
                foreach (WRE2017.RPClthngSeqNumContext context in report.RPClthngSeqNumContextCollection)
                {
                    if (context.IdentifierTFN != report.RPIdentifierTFN)
                    {
                        ProcessMessageDocument processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.001023",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Unexpected TFN mismatch with Reporting Party",
                            LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                            Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                        };
                        response.Add(processMessage);
                    }
                }
            }

            if (report.RPEduSeqNumContextCollection != null)
            {
                foreach (WRE2017.RPEduSeqNumContext context in report.RPEduSeqNumContextCollection)
                {
                    if (context.IdentifierTFN != report.RPIdentifierTFN)
                    {
                        ProcessMessageDocument processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.001023",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Unexpected TFN mismatch with Reporting Party",
                            LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                            Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                        };
                        response.Add(processMessage);
                    }
                }
            }

            if (report.RPOTHWRSeqNumContextCollection != null)
            {
                foreach (WRE2017.RPOTHWRSeqNumContext context in report.RPOTHWRSeqNumContextCollection)
                {
                    if (context.IdentifierTFN != report.RPIdentifierTFN)
                    {
                        ProcessMessageDocument processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.001023",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Unexpected TFN mismatch with Reporting Party",
                            LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                            Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                        };
                        response.Add(processMessage);
                    }
                }
            }

            if (report.RPTrvlSeqNumContextCollection != null)
            {
                foreach (WRE2017.RPTrvlSeqNumContext context in report.RPTrvlSeqNumContextCollection)
                {
                    if (context.IdentifierTFN != report.RPIdentifierTFN)
                    {
                        ProcessMessageDocument processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.001023",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Unexpected TFN mismatch with Reporting Party",
                            LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                            Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023" } },
                        };
                        response.Add(processMessage);
                    }
                }
            }

            return response;

            #endregion
        }
    }
}