﻿using System;
using System.Linq;
using System.Collections.Generic;
using DataContracts;
using System.Text.RegularExpressions;

namespace Ato.EN.IntegrationServices.CodeGenerationIITR
{
    public partial class IITR2018Validator
    {
        protected List<ProcessMessageDocument> ValidateManualRules(IITR2018 report, List<ProcessMessageDocument> response)
        {
            bool assertion;
            ProcessMessageDocument processMessage;

            #region VR.ATO.IITR.000013

            /*  VR.ATO.IITR.000013
            Where the number of days not liable for medicare levy surcharge is less than the whole reporting period, and either income test is triggered; and the reportable employer superannuation contributions is greater than 8% of all income from salary and wages, check reportable employer superannuation contributions as MLS liability may be incurred.
    
            Legacy Rule Format:
            WHERE IN CONTEXT(RP.{SLSPSeqNum})
            IF ([iitr177] = NULL OR [iitr177] < (RP:period.endDate - RP:period.startDate + 1)) AND ((([iitr213] <> NULLORBLANK OR [iitr171] <> NULL) AND ((ConditionalValue([iitr157] > 0, [iitr157], 0) + ConditionalValue(([iitr490] + [iitr491]) > 3772, ([iitr490] + [iitr491]), 0) + ConditionalValue([iitr157] > 0, [iitr330], 0) + [iitr399] - ConditionalValue((([iitr29] > CONVERT_TO_DATE(30,6, [iitr10] - 55)) OR ([iitr29] <= CONVERT_TO_DATE(1,7, [iitr10] - 61))), SUM(ConditionalValue([iitr102] <> TRUE, [iitr99], 0)), 0) + [iitr204] + [iitr205] + [iitr201] + SUM([iitr362]) + [iitr225] + ConditionalValue(([iitr492] + [iitr493]) > 3772, ([iitr492] + [iitr493]), 0) + [iitr227] + [iitr234] + [iitr231] - [iitr236]) > (180000 + ConditionalValue([iitr171] > 1, ((1500 * [iitr171]) - 1), 0)))) OR (([iitr213] = NULLORBLANK) AND ([iitr171] = NULL) AND ((ConditionalValue([iitr157] > 0, [iitr157], 0) + ConditionalValue(([iitr490] + [iitr491]) > 3772, ([iitr490] + [iitr491]), 0) + ConditionalValue([iitr157] > 0, [iitr330], 0) + [iitr399] - ConditionalValue((([iitr29] > CONVERT_TO_DATE(30,6, [iitr10] - 55)) OR ([iitr29] <= CONVERT_TO_DATE(1,7, [iitr10] - 61))), SUM(ConditionalValue([iitr102] <> TRUE, [iitr99], 0)), 0) + [iitr204] + [iitr205] + [iitr201] + SUM([iitr362])) > 90000))) AND ([iitr201] > (SUM([iitr62]) * 0.08))
                RETURN VALIDATION MESSAGE
            ENDIF
        
            Technical Business Rule Format:
            (^IITR177 = NULL OR ^IITR177 < (^EndDate - ^StartDate1 + 1)) AND (((^IITR213 <> BLANK OR ^IITR171 <> NULL) AND ((ConditionalValue(^IITR157 > 0, ^IITR157, 0) + ConditionalValue((^IITR490 + ^IITR491) > 3772, (^IITR490 + ^IITR491), 0) + ConditionalValue(^IITR157 > 0, ^IITR330, 0) + ^IITR399 - ConditionalValue(((^IITR29 > ConvertToDate(30, 6, ^IITR10 - 55)) OR (^IITR29 <= ConvertToDate(1, 7, ^IITR10 - 61))), Sum(ConditionalValue(^IITR102 <> TRUE, ^IITR99, 0)), 0) + ^IITR204 + ^IITR205 + ^IITR201 + Sum(^IITR362) + ^IITR225 + ConditionalValue((^IITR492 + ^IITR493) > 3772, (^IITR492 + ^IITR493), 0) + ^IITR227 + ^IITR234 + ^IITR231 - ^IITR236) > (180000 + ConditionalValue(^IITR171 > 1, ((1500 * ^IITR171) - 1), 0)))) OR ((^IITR213 = BLANK) AND (^IITR171 = NULL) AND ((ConditionalValue(^IITR157 > 0, ^IITR157, 0) + ConditionalValue((^IITR490 + ^IITR491) > 3772, (^IITR490 + ^IITR491), 0) + ConditionalValue(^IITR157 > 0, ^IITR330, 0) + ^IITR399 - ConditionalValue(((^IITR29 > ConvertToDate(30, 6, ^IITR10 - 55)) OR (^IITR29 <= ConvertToDate(1, 7, ^IITR10 - 61))), Sum(ConditionalValue(^IITR102 <> TRUE, ^IITR99, 0)), 0) + ^IITR204 + ^IITR205 + ^IITR201 + Sum(^IITR362)) > 90000))) AND (^IITR201 > (Sum(^IITR62) * 0.08))
            
            Data Elements:
            
            RP:^IITR201 = tns:SuperannuationContribution.EmployerReportable.Amount
            
            RP:^EndDate = RP
            
            RP:^StartDate1 = RP
            
            RP:^IITR177 = tns:IncomeTax.MedicareLevy.SurchargeExemptionDays.Count
            
            RP.SPOUSE:^IITR213 = tns:PersonNameDetails.FamilyName.Text
            
            RP:^IITR171 = tns:IncomeTax.MedicareLevy.DependentChildrenAndStudents.Count
            
            RP:^IITR157 = tns:Income.Taxable.Amount
            
            RP.Eligible:^IITR490 = tns:Income.FringeBenefitsReportable.Amount
            
            RP.Ineligible:^IITR491 = tns:Income.FringeBenefitsReportable.Amount
            
            RP:^IITR330 = tns:InternationalDealings.ExemptForeignEmploymentIncome.Amount
            
            RP:^IITR399 = tns:IncomeTax.FamilyTrustDistributionWhereTaxPaid.Amount
            
            RP:^IITR29 = tns:PersonDemographicDetails.Birth.Date
            
            RP:^IITR10 = tns:Report.TargetFinancial.Year
            
            RP.{SLSPSeqNum}:^IITR102 = tns:Income.Superannuation.DeathBenefitPaymentNonDependant.Indicator
            
            RP.{SLSPSeqNum}:^IITR99 = tns:Income.Superannuation.LumpSumPaymentTaxableComponentTaxedElement.Amount
            
            RP:^IITR204 = tns:Tax.Losses.FinancialInvestmentNet.Amount
            
            RP:^IITR205 = tns:Tax.Losses.RentalPropertyNet.Amount
            
            RP.{SCSeqNum}:^IITR362 = tns:IncomeTax.Deduction.SuperannuationContribution.Amount
            
            RP.SPOUSE:^IITR225 = tns:Income.Taxable.Amount
            
            RP.SPOUSE.Eligible:^IITR492 = tns:Income.FringeBenefitsReportable.Amount
            
            RP.SPOUSE.Ineligible:^IITR493 = tns:Income.FringeBenefitsReportable.Amount
            
            RP.SPOUSE:^IITR227 = tns:IncomeTax.FamilyTrustDistributionWhereTaxPaid.Amount
            
            RP.SPOUSE:^IITR234 = tns:Tax.Losses.InvestmentNet.Amount
            
            RP.SPOUSE:^IITR231 = tns:SuperannuationContribution.EmployerReportable.Amount
            
            RP.SPOUSE:^IITR236 = tns:SuperannuationPayment.BenefitLumpSumZeroTaxRate.Amount
            
            RP.{SaWSeqNum}:^IITR62 = tns:Income.SalaryOrWages.Amount
            */
            assertion = ((report.IITR177 == null || Convert.ToDouble(report.IITR177.GetValueOrDefault()) < (report.RPEndDate - report.RPStartDate).GetValueOrDefault().TotalDays + 1) && (((!string.IsNullOrWhiteSpace(report.IITR213) || report.IITR171 != null) && ((report.IITR157.GetValueOrDefault() > 0 ? report.IITR157.GetValueOrDefault() : 0) + ((report.IITR490.GetValueOrDefault() + report.IITR491.GetValueOrDefault()) > 3772 ? (report.IITR490.GetValueOrDefault() + report.IITR491.GetValueOrDefault()) : 0) + (report.IITR157.GetValueOrDefault() > 0 ? report.IITR330.GetValueOrDefault() : 0) + report.IITR399.GetValueOrDefault() - (report.IITR29 > ConvertToDate(30, 6, Convert.ToInt32(report.IITR10.GetValueOrDefault()) - 55) || report.IITR29 <= ConvertToDate(1, 7, Convert.ToInt32(report.IITR10.GetValueOrDefault()) - 61) ? (report.RPSLSPSeqNumContextCollection != null ? report.RPSLSPSeqNumContextCollection.FindAll(f => f.IITR102 != true).Sum(f => f.IITR99.GetValueOrDefault()) : 0) : 0) + report.IITR204.GetValueOrDefault() + report.IITR205.GetValueOrDefault() + report.IITR201.GetValueOrDefault() + (report.RPSCSeqNumContextCollection != null ? report.RPSCSeqNumContextCollection.Sum(f => f.IITR362.GetValueOrDefault()) : 0) + report.IITR225.GetValueOrDefault() + (report.IITR492.GetValueOrDefault() + report.IITR493.GetValueOrDefault() > 3772 ? report.IITR492.GetValueOrDefault() + report.IITR493.GetValueOrDefault() : 0) + report.IITR227.GetValueOrDefault() + report.IITR234.GetValueOrDefault() + report.IITR231.GetValueOrDefault() - report.IITR236.GetValueOrDefault()) > (180000 + (report.IITR171.GetValueOrDefault() > 1 ? 1500 * report.IITR171.GetValueOrDefault() - 1 : 0))) || (string.IsNullOrWhiteSpace(report.IITR213) && report.IITR171 == null && ((report.IITR157.GetValueOrDefault() > 0 ? report.IITR157.GetValueOrDefault() : 0) + (report.IITR490.GetValueOrDefault() + report.IITR491.GetValueOrDefault() > 3772 ? report.IITR490.GetValueOrDefault() + report.IITR491.GetValueOrDefault() : 0) + (report.IITR157.GetValueOrDefault() > 0 ? report.IITR330.GetValueOrDefault() : 0) + report.IITR399.GetValueOrDefault() - (report.IITR29 > ConvertToDate(30, 6, Convert.ToInt32(report.IITR10) - 55) || report.IITR29 <= ConvertToDate(1, 7, Convert.ToInt32(report.IITR10) - 61) ? (report.RPSLSPSeqNumContextCollection != null ? report.RPSLSPSeqNumContextCollection.FindAll(f => f.IITR102 != true).Sum(f => f.IITR99.GetValueOrDefault()) : 0) : 0) + report.IITR204.GetValueOrDefault() + report.IITR205.GetValueOrDefault() + report.IITR201.GetValueOrDefault() + (report.RPSCSeqNumContextCollection != null ? report.RPSCSeqNumContextCollection.Sum(f => f.IITR362.GetValueOrDefault()) : 0)) > 90000)) && (report.RPSaWSeqNumContextCollection != null ? Convert.ToDouble(report.IITR201.GetValueOrDefault()) > Convert.ToDouble(report.RPSaWSeqNumContextCollection.Sum(f => f.IITR62.GetValueOrDefault())) * 0.08 : false));

            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.000013",
                    Severity = ProcessMessageSeverity.Warning,
                    Description = @"Check reportable employer superannuation contributions as MLS liability may be incurred",
                    LongDescription = @"Check that compulsory super contributions have not been included in reportable employer superannuation contributions at the income test section. Some reportable employer superannuation contributions are incorrectly reported on payment summaries, these amounts are generally salary sacrifice super contributions only. Incorrect reporting may cause a MLS liability.",
                    Location = "/xbrli:xbrl/tns:SuperannuationContribution.EmployerReportable.Amount[@contextRef='" + report.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.000013" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR177", Value = report.IITR177.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR213", Value = report.IITR213 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR171", Value = report.IITR171.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR157", Value = report.IITR157.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR490", Value = report.IITR490.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR491", Value = report.IITR491.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR330", Value = report.IITR330.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR399", Value = report.IITR399.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR29", Value = GetValueOrEmpty(report.IITR29) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR10", Value = report.IITR10.GetValueOrDefault().ToString() });
                
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR99", Value = (report.RPSLSPSeqNumContextCollection != null ? report.RPSLSPSeqNumContextCollection.FindAll(f => f.IITR102 != true).Sum(f => f.IITR99).ToString() : "null") });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR204", Value = report.IITR204.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR205", Value = report.IITR205.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR201", Value = report.IITR201.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR362", Value = (report.RPSCSeqNumContextCollection != null ? report.RPSCSeqNumContextCollection.Sum(f => f.IITR362).ToString() : "null") });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR225", Value = report.IITR225.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR492", Value = report.IITR492.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR493", Value = report.IITR493.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR227", Value = report.IITR227.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR234", Value = report.IITR234.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR231", Value = report.IITR231.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR236", Value = report.IITR236.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR62", Value = (report.RPSaWSeqNumContextCollection != null ? report.RPSaWSeqNumContextCollection.Sum(f => f.IITR62).ToString() : "null") });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.000013

            #region VR.ATO.IITR.000265

            /*  VR.ATO.IITR.000265
            Where a taxpayer is claiming an amount for net foreign employment income, there must be an amount at Salary and Wages income.

            Legacy Rule Format:
            IF ([iitr329] <> NULL) AND ALL OCCURRENCES OF ([iitr62] = 0 OR [iitr62] = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IITR329 <> NULL) AND  (^IITR62 = 0 OR ^IITR62 = NULL)

            Data Elements:

            RP:^IITR329 = tns:InternationalDealings.ForeignEmploymentIncomePaymentSummary.Amount

            RP.{SaWSeqNum}:^IITR62 = tns:Income.SalaryOrWages.Amount
            */
            assertion = (report.IITR329 != null && (report.RPSaWSeqNumContextCollection != null ? report.RPSaWSeqNumContextCollection.Exists(context => context.IITR62 != 0 && context.IITR62 != null) == false : true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.000265",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Salary and wage income must be greater than $0 where net foreign employment income is reported.",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.ForeignEmploymentIncomePaymentSummary.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.000265" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR329", Value = report.IITR329.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR62", Value = (report.RPSaWSeqNumContextCollection != null ? report.RPSaWSeqNumContextCollection.Sum(f => f.IITR62).ToString() : "null") });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.000265

            #region VR.ATO.IITR.100002

            /*  VR.ATO.IITR.100002
            The Australian superannuation lump sum payments "Date of payment" must be within the current financial year or approved substitute period.
    
            Legacy Rule Format:
            IF (ANY OCCURRENCE OF ([iitr97]) < RP:period.startDate) OR (ANY OCCURRENCE OF ([iitr97]) > RP:period.endDate)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (AnyOccurrence(^IITR97, ^IITR97 < ^StartDate1)) OR (AnyOccurrence(^IITR97, ^IITR97 > ^EndDate))
    
            Data Elements:
    
            RP.{SLSPSeqNum}:^IITR97 = tns:Income.Superannuation.LumpSumPayment.Date
    
            RP.{SLSPSeqNum}:^EndDate = RP
    
            RP.{SLSPSeqNum}:^StartDate1 = RP
            */
            assertion = (report.RPSLSPSeqNumContextCollection != null ? report.RPSLSPSeqNumContextCollection.Any(IITR97Repeat => IITR97Repeat.IITR97 < report.RPStartDate) || report.RPSLSPSeqNumContextCollection.Any(IITR97Repeat => IITR97Repeat.IITR97 > report.RPEndDate) : false);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.100002",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"SLS date of payment must be within income year.",
                    Location = "/xbrli:xbrl/tns:Income.Superannuation.LumpSumPayment.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.100002" } },
                };

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.100002

            #region VR.ATO.IITR.100003

            /*  VR.ATO.IITR.100003
            The ETP payment date must be within the current financial year or approved substitute period.
    
            Legacy Rule Format:
            IF (ANY OCCURRENCE OF ([iitr73]) < RP:period.startDate) OR (ANY OCCURRENCE OF ([iitr73]) > RP:period.endDate)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (AnyOccurrence(^IITR73, ^IITR73 < ^StartDate1)) OR (AnyOccurrence(^IITR73, ^IITR73 > ^EndDate))
    
            Data Elements:
    
            RP.{ETPSeqNum}:^IITR73 = tns:Income.EmploymentTerminationPayment.Date
    
            RP.{ETPSeqNum}:^EndDate = RP
    
            RP.{ETPSeqNum}:^StartDate1 = RP
            */
            assertion = (report.RPETPSeqNumContextCollection != null ? report.RPETPSeqNumContextCollection.Any(IITR73Repeat => IITR73Repeat.IITR73 < report.RPStartDate) || report.RPETPSeqNumContextCollection.Any(IITR73Repeat => IITR73Repeat.IITR73 > report.RPEndDate) : false);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.100003",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"ETP date of payment must be within income year.",
                    Location = "/xbrli:xbrl/tns:Income.EmploymentTerminationPayment.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.100003" } },
                };

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.100003

            #region VR.ATO.IITR.100043

            /*  VR.ATO.IITR.100043
            Adjusted taxable income must equal the sum of Taxable income or loss (when amount above $0), Reportable employer superannuation contributions, Tax-free government pensions, Target foreign income, Net financial investment loss, Net rental property loss, and all Deduction claimed amounts, plus Total reportable fringe benefits amount from employers not exempt under section 57A and 53.0% of Total reportable fringe benefits amount from employers exempt under section 57A (when combined Total reportable fringe benefits amount above $3,772), less Child support you paid. If you are completing a tax return for a deceased person, this value must be divided by the number of days the taxpayer was alive during the financial year and multiplied by the total number of days in the financial year.

            Legacy Rule Format:
            IF (([iitr28] = NULL OR [iitr28] > RP:period.endDate) AND (([iitr208] <> NULL AND [iitr208] <> ((ConditionalValue([iitr157] > 0, [iitr157], 0) + ConditionalValue(([iitr490] + [iitr491]) > 3772, (([iitr490] * 0.530) + [iitr491]), 0) + [iitr201] + [iitr202] + [iitr203] + [iitr204] + [iitr205] + SUM([iitr362]) - [iitr206]) +/- 1)) OR ([iitr208] = NULL AND ((ConditionalValue([iitr157] > 0, [iitr157], 0) + ConditionalValue(([iitr490] + [iitr491]) > 3772, (([iitr490] * 0.530) + [iitr491]), 0) + [iitr201] + [iitr202] + [iitr203] + [iitr204] + [iitr205] + SUM([iitr362]) - [iitr206]) +/- 1) <> 0))) OR (([iitr28] > (RP:period.startDate - 1) AND [iitr28] < (RP:period.endDate + 1)) AND (([iitr208] <> NULL AND [iitr208] <> (((ConditionalValue([iitr157] > 0, [iitr157], 0) + ConditionalValue(([iitr490] + [iitr491]) > 3772, (([iitr490] * 0.530) + [iitr491]), 0) + [iitr201] + [iitr202] + [iitr203] + [iitr204] + [iitr205] + SUM([iitr362]) - [iitr206]) * (RP:period.endDate - RP:period.startDate + 1) / ([iitr28] - RP:period.startDate + 1)) +/- 1)) OR ([iitr208] = NULL AND ((((ConditionalValue([iitr157] > 0, [iitr157], 0) + ConditionalValue(([iitr490] + [iitr491]) > 3772, (([iitr490] * 0.530) + [iitr491]), 0) + [iitr201] + [iitr202] + [iitr203] + [iitr204] + [iitr205] + SUM([iitr362]) - [iitr206]) * (RP:period.endDate - RP:period.startDate + 1) / ([iitr28] - RP:period.startDate + 1)) +/- 1) <> 0)))) OR ([iitr28] < RP:period.startDate AND [iitr208] <> 0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ((^IITR28 = NULL OR ^IITR28 > ^EndDate) AND ((^IITR208 <> NULL AND ^IITR208 <> ((ConditionalValue(^IITR157 > 0, ^IITR157, 0) + ConditionalValue((^IITR490 + ^IITR491) > 3772, ((^IITR490 * 0.530) + ^IITR491), 0) + ^IITR201 + ^IITR202 + ^IITR203 + ^IITR204 + ^IITR205 + Sum(^IITR362) - ^IITR206) +/- 1)) OR (^IITR208 = NULL AND ((ConditionalValue(^IITR157 > 0, ^IITR157, 0) + ConditionalValue((^IITR490 + ^IITR491) > 3772, ((^IITR490 * 0.530) + ^IITR491), 0) + ^IITR201 + ^IITR202 + ^IITR203 + ^IITR204 + ^IITR205 + Sum(^IITR362) - ^IITR206) +/- 1) <> 0))) OR ((^IITR28 > (^StartDate1 - 1) AND ^IITR28 < (^EndDate + 1)) AND ((^IITR208 <> NULL AND ^IITR208 <> (((ConditionalValue(^IITR157 > 0, ^IITR157, 0) + ConditionalValue((^IITR490 + ^IITR491) > 3772, ((^IITR490 * 0.530) + ^IITR491), 0) + ^IITR201 + ^IITR202 + ^IITR203 + ^IITR204 + ^IITR205 + Sum(^IITR362) - ^IITR206) * (^EndDate - ^StartDate2 + 1) / (^IITR28 - ^StartDate3 + 1)) +/- 1)) OR (^IITR208 = NULL AND ((((ConditionalValue(^IITR157 > 0, ^IITR157, 0) + ConditionalValue((^IITR490 + ^IITR491) > 3772, ((^IITR490 * 0.530) + ^IITR491), 0) + ^IITR201 + ^IITR202 + ^IITR203 + ^IITR204 + ^IITR205 + Sum(^IITR362) - ^IITR206) * (^EndDate - ^StartDate4 + 1) / (^IITR28 - ^StartDate5 + 1)) +/- 1) <> 0)))) OR (^IITR28 < ^StartDate6 AND ^IITR208 <> 0)

            Data Elements:

            RP:^IITR208 = tns:Income.TaxableAdjusted.Amount

            RP:^EndDate = RP

            RP:^StartDate1 = RP

            RP:^StartDate2 = RP

            RP:^StartDate3 = RP

            RP:^StartDate4 = RP

            RP:^StartDate5 = RP

            RP:^StartDate6 = RP

            RP:^IITR28 = tns:PersonDemographicDetails.Death.Date

            RP:^IITR157 = tns:Income.Taxable.Amount

            RP.Eligible:^IITR490 = tns:Income.FringeBenefitsReportable.Amount

            RP.Ineligible:^IITR491 = tns:Income.FringeBenefitsReportable.Amount

            RP:^IITR201 = tns:SuperannuationContribution.EmployerReportable.Amount

            RP:^IITR202 = tns:Pension.TaxFree.Amount

            RP:^IITR203 = tns:InternationalDealings.ForeignIncomeTarget.Amount

            RP:^IITR204 = tns:Tax.Losses.FinancialInvestmentNet.Amount

            RP:^IITR205 = tns:Tax.Losses.RentalPropertyNet.Amount

            RP.{SCSeqNum}:^IITR362 = tns:IncomeTax.Deduction.SuperannuationContribution.Amount

            RP:^IITR206 = tns:ChildSupport.Payment.Amount
            */
            double calc1_100043 = (report.IITR157.GetValueOrDefault() > 0 ? Convert.ToDouble(report.IITR157.GetValueOrDefault()) : 0) + (report.IITR490.GetValueOrDefault() + report.IITR491.GetValueOrDefault() > 3772 ? Math.Floor((Convert.ToDouble(report.IITR490.GetValueOrDefault()) * 0.53) + Convert.ToDouble(report.IITR491.GetValueOrDefault())) : 0) + Convert.ToDouble(report.IITR201.GetValueOrDefault() + report.IITR202.GetValueOrDefault() + report.IITR203.GetValueOrDefault() + report.IITR204.GetValueOrDefault() + report.IITR205.GetValueOrDefault() + (report.RPSCSeqNumContextCollection != null ? report.RPSCSeqNumContextCollection.Sum(f => f.IITR362.GetValueOrDefault()) : 0) - report.IITR206.GetValueOrDefault());
            double calc2_100043 = (report.IITR28 != null ? calc1_100043 * (((report.RPEndDate - report.RPStartDate).Value.TotalDays + 1) / ((Convert.ToDateTime(report.IITR28) - report.RPStartDate).Value.TotalDays + 1)) : 0);

            assertion = (((report.IITR28 == null || Convert.ToDateTime(report.IITR28) > report.RPEndDate) && (((report.IITR208 != null && Convert.ToDouble(report.IITR208) < (calc1_100043 - 1) || Convert.ToDouble(report.IITR208) > (calc1_100043 + 1))) || (report.IITR208 == null && (0 < (calc1_100043 - 1) || 0 > (calc1_100043 + 1))))) || (report.IITR28 != null ? ((Convert.ToDateTime(report.IITR28) > report.RPStartDate.Value.AddDays(-1) && Convert.ToDateTime(report.IITR28) < report.RPEndDate.Value.AddDays(1)) && ((report.IITR208 != null && (Convert.ToDouble(report.IITR208.GetValueOrDefault()) < (calc2_100043 - 1) || Convert.ToDouble(report.IITR208.GetValueOrDefault()) > (calc2_100043 + 1))) || report.IITR208 == null && (0 < (calc2_100043 - 1) || 0 > (calc2_100043 + 1)))) : false) || ((report.IITR28 != null ? report.IITR28 < report.RPStartDate : false) && (report.IITR208 != null ? report.IITR208 : 0) != 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.023015",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Adjusted taxable income is incorrect",
                    LongDescription = @"Adjusted taxable income must equal the sum of Taxable income or loss (when amount above $0), Reportable employer superannuation contributions, Tax-free government pensions, Target foreign income, Net financial investment loss, Net rental property loss, and all Deduction claimed amounts, plus Total reportable fringe benefits amount from employers not exempt under section 57A and 53.0% of Total reportable fringe benefits amount from employers exempt under section 57A (when combined Total reportable fringe benefits amount above $3,772), less Child support you paid. If you are completing a tax return for a deceased person, this value must be divided by the number of days the taxpayer was alive during the financial year and multiplied by the total number of days in the financial year.",
                    Location = "/xbrli:xbrl/tns:Income.TaxableAdjusted.Amount[@contextRef='" + report.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.100043" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR28", Value = GetValueOrEmpty(report.IITR28) });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR157", Value = report.IITR157.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR201", Value = report.IITR201.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR202", Value = report.IITR202.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR203", Value = report.IITR203.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR204", Value = report.IITR204.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR205", Value = report.IITR205.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR206", Value = report.IITR206.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR208", Value = report.IITR208.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR362", Value = (report.RPSCSeqNumContextCollection != null ? report.RPSCSeqNumContextCollection.Sum(f => f.IITR362.GetValueOrDefault()).ToString() : "null") });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR490", Value = report.IITR490.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR491", Value = report.IITR491.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "StartDate", Value = GetValueOrEmpty(report.RPStartDate) });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "EndDate", Value = GetValueOrEmpty(report.RPEndDate) });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.100043

            #region VR.ATO.IITR.200014

            /*  VR.ATO.IITR.200014
            Where the 'Seniors and pensioners - Tax offset code' indicates at any time during the year the taxpayer was single or widowed, or separated; and all occurrences of Deduction claimed amounts are null, check deductible personal superannuation contributions.

            Legacy Rule Format:
            IF ([iitr163] = "Single") AND ALL OCCURRENCES OF([iitr362]) = NULL
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IITR163 = 'Single') AND ^IITR362 = NULL

            Data Elements:

            RP:^IITR163 = tns:TaxOffsetClaim.SeniorAndPensioner.Code

            RP.{SCSeqNum}:^IITR362 = tns:IncomeTax.Deduction.SuperannuationContribution.Amount
            */
            assertion = (report.IITR163 == @"Single" && (report.RPSCSeqNumContextCollection != null ? report.RPSCSeqNumContextCollection.Exists(context => context.IITR362 != null) == false : true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.200014",
                    Severity = ProcessMessageSeverity.Warning,
                    Description = @"Check deductible personal superannuation contributions",
                    LongDescription = @"Rebate income is used to determine eligibility for the Seniors and pensioners tax offset. Where applicable, complete Deductible personal superannuation contributions from the Supplementary section.",
                    Location = "/xbrli:xbrl/tns:TaxOffsetClaim.SeniorAndPensioner.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.200014" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR163", Value = report.IITR163 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR362", Value = "null" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.200014

            #region VR.ATO.IITR.200033

            /*  VR.ATO.IITR.200033
            Where the taxpayer was not covered by private hospital cover for the whole reporting period, the number of days not liable for medicare surcharge was less than the whole reporting period, and no occurrences of Deduction claimed (deductible personal superannuation contributions) amounts are present, check Deduction claimed amounts

            Legacy Rule Format:
            IF ([iitr176] <> TRUE) AND ([iitr177] < (RP:Period.endDate - RP:Period.startDate + 1)) AND ALL OCCURRENCES OF([iitr362]) = NULL
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IITR176 <> TRUE) AND (^IITR177 < (DaysInDuration(^EndDate - ^StartDate1) + 1)) AND ^IITR362 = NULL

            Data Elements:

            RP.{SCSeqNum}:^IITR362 = tns:IncomeTax.Deduction.SuperannuationContribution.Amount

            RP.{SCSeqNum}:^EndDate = RP

            RP.{SCSeqNum}:^StartDate1 = RP

            RP:^IITR176 = tns:IncomeTax.MedicareLevy.PrivateHospitalCoverWholePeriod.Indicator

            RP:^IITR177 = tns:IncomeTax.MedicareLevy.SurchargeExemptionDays.Count
            */
            assertion = ((report.RPExists == true) && (report.IITR176 != true && report.IITR177.GetValueOrDefault() < (report.RPEndDate.GetValueOrDefault() - report.RPStartDate.GetValueOrDefault()).Days + 1 && (report.RPSCSeqNumContextCollection != null ? report.RPSCSeqNumContextCollection.Exists(context => context.IITR362 != null) == false : true)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.200033",
                    Severity = ProcessMessageSeverity.Warning,
                    Description = @"Check deductible personal superannuation contributions",
                    LongDescription = @"Income for surcharge purposes is used to determine if the taxpayer is liable for Medicare Levy Surcharge. Where applicable, complete Deductible personal superannuation contributions from the Supplementary section.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.SuperannuationContribution.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.200033" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR176", Value = GetValueOrEmpty(report.IITR176) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR177", Value = report.IITR177.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR362", Value = "null" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.200033

            #region VR.ATO.IITR.200034

            /*  VR.ATO.IITR.200034
            Spouse's or de facto spouse's family name is present and the taxpayer had a spouse for the full year and had no Medicare levy surcharge. Combined Total reportable fringe benefits amounts exceed $3,772 and taxable income or loss plus total reportable fringe benefits amounts plus net amount on which family trust has been paid plus exempt foreign employment income exceeds $21,980 and number of days not liable for MLS, spouse income details also required.
    
            Legacy Rule Format:
            IF ([iitr213] <> NULLORBLANK OR [iitr220] = TRUE) AND [iitr176] = FALSE AND ([iitr157] + ConditionalValue(([iitr490] + [iitr491]) > 3772, ([iitr490] + [iitr491]), 0) + ConditionalValue([iitr157] > 0, [iitr330], 0) + [iitr399] - ConditionalValue(ANY OCCURRENCE OF([iitr102]) <> TRUE AND [iitr29] <> NULL AND ([iitr29] > ConvertToDate(30,06,([iitr10] - 55)) OR [iitr29] <= ConvertToDate(01,07,([iitr10] - 61))), SUM([iitr99]), 0) + [iitr204] + [iitr205] + [iitr201] + SUM([iitr362])) > 21980 AND ([iitr177] = NULL OR ([iitr225] = NULL AND [iitr226] = NULL AND [iitr227] = NULL AND [iitr492] = NULL AND [iitr493] = NULL AND [iitr231] = NULL AND [iitr234] = NULL AND [iitr236] = NULL))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IITR213 <> BLANK OR ^IITR220 = TRUE) AND ^IITR176 = FALSE AND (^IITR157 + ConditionalValue((^IITR490 + ^IITR491) > 3772, (^IITR490 + ^IITR491), 0) + ConditionalValue(^IITR157 > 0, ^IITR330, 0) + ^IITR399 - ConditionalValue(AnyOccurrence(^IITR102, ^IITR102 <> TRUE) AND (^IITR29 <> NULL AND ^IITR29 > ConvertToDate(30, 06, (^IITR10 - 55)) OR ^IITR29 <= ConvertToDate(01, 07, (^IITR10 - 61))), Sum(^IITR99), 0) + ^IITR204 + ^IITR205 + ^IITR201 + Sum(^IITR362)) > 21980 AND (^IITR177 = NULL OR (^IITR225 = NULL AND ^IITR226 = NULL AND ^IITR227 = NULL AND ^IITR492 = NULL AND ^IITR493 = NULL AND ^IITR231 = NULL AND ^IITR234 = NULL AND ^IITR236 = NULL))
    
            Data Elements:
    
            RP:^IITR177 = tns:IncomeTax.MedicareLevy.SurchargeExemptionDays.Count
    
            RP.SPOUSE:^IITR213 = tns:PersonNameDetails.FamilyName.Text
    
            RP:^IITR220 = tns:PersonDemographicDetails.SpouseFullYear.Indicator
    
            RP:^IITR176 = tns:IncomeTax.MedicareLevy.PrivateHospitalCoverWholePeriod.Indicator
    
            RP:^IITR157 = tns:Income.Taxable.Amount
    
            RP.Eligible:^IITR490 = tns:Income.FringeBenefitsReportable.Amount
    
            RP.Ineligible:^IITR491 = tns:Income.FringeBenefitsReportable.Amount
    
            RP:^IITR330 = tns:InternationalDealings.ExemptForeignEmploymentIncome.Amount
    
            RP:^IITR399 = tns:IncomeTax.FamilyTrustDistributionWhereTaxPaid.Amount
    
            RP.{SLSPSeqNum}:^IITR102 = tns:Income.Superannuation.DeathBenefitPaymentNonDependant.Indicator
    
            RP:^IITR29 = tns:PersonDemographicDetails.Birth.Date
    
            RP:^IITR10 = tns:Report.TargetFinancial.Year
    
            RP.{SLSPSeqNum}:^IITR99 = tns:Income.Superannuation.LumpSumPaymentTaxableComponentTaxedElement.Amount
    
            RP:^IITR204 = tns:Tax.Losses.FinancialInvestmentNet.Amount
    
            RP:^IITR205 = tns:Tax.Losses.RentalPropertyNet.Amount
    
            RP:^IITR201 = tns:SuperannuationContribution.EmployerReportable.Amount
    
            RP.{SCSeqNum}:^IITR362 = tns:IncomeTax.Deduction.SuperannuationContribution.Amount
    
            RP.SPOUSE:^IITR225 = tns:Income.Taxable.Amount
    
            RP.SPOUSE:^IITR226 = tns:IncomeTax.TrustIncomeNotIncludedInTaxableIncome.Amount
    
            RP.SPOUSE:^IITR227 = tns:IncomeTax.FamilyTrustDistributionWhereTaxPaid.Amount
    
            RP.SPOUSE.Eligible:^IITR492 = tns:Income.FringeBenefitsReportable.Amount
    
            RP.SPOUSE.Ineligible:^IITR493 = tns:Income.FringeBenefitsReportable.Amount
    
            RP.SPOUSE:^IITR231 = tns:SuperannuationContribution.EmployerReportable.Amount
    
            RP.SPOUSE:^IITR234 = tns:Tax.Losses.InvestmentNet.Amount
    
            RP.SPOUSE:^IITR236 = tns:SuperannuationPayment.BenefitLumpSumZeroTaxRate.Amount
            */
            assertion = ((string.IsNullOrWhiteSpace(report.IITR213) != true || report.IITR220 == true) && report.IITR176 == false && report.IITR157.GetValueOrDefault() + ConditionalValue(report.IITR490.GetValueOrDefault() + report.IITR491.GetValueOrDefault() > 3772, report.IITR490.GetValueOrDefault() + report.IITR491.GetValueOrDefault(), 0) + ConditionalValue(report.IITR157.GetValueOrDefault() > 0, report.IITR330.GetValueOrDefault(), 0) + report.IITR399.GetValueOrDefault() - ConditionalValue((report.RPSLSPSeqNumContextCollection != null ? report.RPSLSPSeqNumContextCollection.Any(IITR102Repeat => IITR102Repeat.IITR102 != true) : false) && (report.IITR29 != null && report.IITR29.GetValueOrDefault() > ConvertToDate(30, 6, report.IITR10.GetValueOrDefault() - 55) || report.IITR29.GetValueOrDefault() <= ConvertToDate(1, 7, report.IITR10.GetValueOrDefault() - 61)), (report.RPSLSPSeqNumContextCollection != null ? report.RPSLSPSeqNumContextCollection.Sum(f => f.IITR99.GetValueOrDefault()) : 0), 0) + report.IITR204.GetValueOrDefault() + report.IITR205.GetValueOrDefault() + report.IITR201.GetValueOrDefault() + (report.RPSCSeqNumContextCollection != null ? report.RPSCSeqNumContextCollection.Sum(f => f.IITR362.GetValueOrDefault()) : 0) > 21980 && (report.IITR177 == null || report.IITR225 == null && report.IITR226 == null && report.IITR227 == null && report.IITR492 == null && report.IITR493 == null && report.IITR231 == null && report.IITR234 == null && report.IITR236 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.201034",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of days not liable for surcharge or spouse income details is required",
                    LongDescription = @"Spouse's or de facto spouse's family name is present and the taxpayer had a spouse for the full year and had no Medicare levy surcharge. Combined Total reportable fringe benefits amounts exceed $3,772 and taxable income or loss plus total reportable fringe benefits amounts plus net amount on which family trust has been paid plus exempt foreign employment income exceeds $21,980 and number of days not liable for MLS, spouse income details also required.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.MedicareLevy.SurchargeExemptionDays.Count",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.200034" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR213", Value = report.IITR213 });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR220", Value = GetValueOrEmpty(report.IITR220) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR176", Value = GetValueOrEmpty(report.IITR176) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR157", Value = report.IITR157.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR490", Value = report.IITR490.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR491", Value = report.IITR491.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR330", Value = report.IITR330.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR399", Value = report.IITR399.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR102", Value = (report.RPSLSPSeqNumContextCollection != null ? report.RPSLSPSeqNumContextCollection.Any(IITR102Repeat => IITR102Repeat.IITR102 != true).ToString() : "false") });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR29", Value = GetValueOrEmpty(report.IITR29) });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR10", Value = report.IITR10.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR99", Value = (report.RPSLSPSeqNumContextCollection != null ? report.RPSLSPSeqNumContextCollection.Sum(f => f.IITR99.GetValueOrDefault()).ToString() : "0") });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR204", Value = report.IITR204.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR205", Value = report.IITR205.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR201", Value = report.IITR201.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR362", Value = (report.RPSCSeqNumContextCollection != null ? report.RPSCSeqNumContextCollection.Sum(f => f.IITR362).ToString() : "null") });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR177", Value = report.IITR177.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR225", Value = report.IITR225.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR226", Value = report.IITR226.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR227", Value = report.IITR227.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR492", Value = report.IITR492.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR493", Value = report.IITR493.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR231", Value = report.IITR231.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR234", Value = report.IITR234.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR236", Value = report.IITR236.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.200034

            #region VR.ATO.IITR.300013

            /*  VR.ATO.IITR.300013
            Where no income and deduction fields are provided, check if this is a nil return.

            Legacy Rule Format:
            IF (ALL OCCURRENCES OF([iitr62]) = NULL) AND ([iitr65] = NULL) AND (ALL OCCURRENCES OF([iitr68]) = NULL) AND (ALL OCCURRENCES OF([iitr71]) = NULL) AND ([iitr86] = NULL) AND ([iitr89] = NULL) AND ([iitr92] = NULL) AND ([iitr93] = NULL) AND ([iitr505] = NULL) AND ([iitr94] = NULL) AND ([iitr95] = NULL) AND ([iitr115] = NULL) AND ([iitr118] = NULL) AND ([iitr119] = NULL) AND ([iitr120] = NULL) AND ([iitr134] = NULL) AND ([iitr136] = NULL) AND ([iitr137] = NULL) AND ([iitr139] = NULL) AND ([iitr141] = NULL) AND ([iitr148] = NULL) AND ([iitr502] = NULL) AND ([iitr503] = NULL) AND ([iitr504] = NULL) AND ([iitr154] = NULL) AND ([iitr156] = NULL) AND ([iitr242] = NULL) AND ([iitr243] = NULL) AND ([iitr245] = NULL) AND ([iitr246] = NULL) AND ([iitr252] = NULL) AND ([iitr255] = NULL) AND ([iitr259] = NULL) AND ([iitr260] = NULL) AND ([iitr265] = NULL) AND ([iitr281] = NULL) AND ([iitr285] = NULL) AND ([iitr306] = NULL) AND ([iitr311] = NULL) AND ([iitr316] = NULL) AND ([iitr318] = NULL) AND ([iitr323] = NULL) AND ([iitr327] = NULL) AND ([iitr334] = NULL) AND ([iitr335] = NULL) AND ([iitr337] = NULL) AND ([iitr339] = NULL) AND (ALL OCCURRENCES OF([iitr348]) = NULL) AND (ALL OCCURRENCES OF([iitr561]) = NULL) AND (ALL OCCURRENCES OF([iitr566]) = NULL) AND ([iitr358] = NULL) AND (ALL OCCURRENCES OF([iitr362]) = NULL) AND ([iitr370] = NULL) AND (ALL OCCURRENCES OF([iitr375]) = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IITR62 = NULL) AND (^IITR65 = NULL) AND (^IITR68 = NULL) AND (^IITR71 = NULL) AND (^IITR86 = NULL) AND (^IITR89 = NULL) AND (^IITR92 = NULL) AND (^IITR93 = NULL) AND (^IITR505 = NULL) AND (^IITR94 = NULL) AND (^IITR95 = NULL) AND (^IITR115 = NULL) AND (^IITR118 = NULL) AND (^IITR119 = NULL) AND (^IITR120 = NULL) AND (^IITR134 = NULL) AND (^IITR136 = NULL) AND (^IITR137 = NULL) AND (^IITR139 = NULL) AND (^IITR141 = NULL) AND (^IITR148 = NULL) AND (^IITR502 = NULL) AND (^IITR503 = NULL) AND (^IITR504 = NULL) AND (^IITR154 = NULL) AND (^IITR156 = NULL) AND (^IITR242 = NULL) AND (^IITR243 = NULL) AND (^IITR245 = NULL) AND (^IITR246 = NULL) AND (^IITR252 = NULL) AND (^IITR255 = NULL) AND (^IITR259 = NULL) AND (^IITR260 = NULL) AND (^IITR265 = NULL) AND (^IITR281 = NULL) AND (^IITR285 = NULL) AND (^IITR306 = NULL) AND (^IITR311 = NULL) AND (^IITR316 = NULL) AND (^IITR318 = NULL) AND (^IITR323 = NULL) AND (^IITR327 = NULL) AND (^IITR334 = NULL) AND (^IITR335 = NULL) AND (^IITR337 = NULL) AND (^IITR339 = NULL) AND (^IITR348 = NULL) AND (^IITR561 = NULL) AND (^IITR566 = NULL) AND (^IITR358 = NULL) AND (^IITR362 = NULL) AND (^IITR370 = NULL) AND (^IITR375 = NULL)

            Data Elements:

            RP.{SaWSeqNum}:^IITR62 = tns:Income.SalaryOrWages.Amount

            RP:^IITR65 = tns:Income.AllowancesEarningsTipsDirectorsFees.Amount

            RP.{EmpLSPSeqNum}:^IITR68 = tns:Income.UnusedAnnualOrLongServiceLeavePaymentLumpSumA.Amount

            RP.{EmpLSPSeqNum}:^IITR71 = tns:Income.UnusedAnnualOrLongServiceLeavePaymentLumpSumB.Amount

            RP:^IITR86 = tns:Income.AustralianGovernmentAllowancesAndPayments.Amount

            RP:^IITR89 = tns:Pension.Total.Amount

            RP:^IITR92 = tns:Income.Superannuation.AnnuitiesAndSuperannuationTaxableComponentTaxedElement.Amount

            RP:^IITR93 = tns:Income.Superannuation.AnnuitiesAndSuperannuationTaxableComponentUntaxedElement.Amount
            
            RP:^IITR505 = tns:Income.Superannuation.AnnuitiesAndSuperannuationAssessableCappedBenefit.Amount

            RP:^IITR94 = tns:Income.Superannuation.AnnuitiesAndSuperannuationLumpSumArrearsTaxableComponentTaxedElement.Amount

            RP:^IITR95 = tns:Income.Superannuation.AnnuitiesAndSuperannuationLumpSumArrearsTaxableComponentUntaxedElement.Amount
            
            RP:^IITR115 = tns:Income.Interest.Gross.Amount

            RP:^IITR118 = tns:Income.DividendsUnfranked.Amount

            RP:^IITR119 = tns:Income.DividendsFranked.Amount

            RP:^IITR120 = tns:IncomeTax.FrankingCredits.Amount

            RP:^IITR134 = tns:IncomeTax.Deduction.Car.Amount

            RP:^IITR136 = tns:IncomeTax.Deduction.Travel.Amount

            RP:^IITR137 = tns:IncomeTax.Deduction.Clothing.Amount

            RP:^IITR139 = tns:IncomeTax.Deduction.EducationNet.Amount

            RP:^IITR141 = tns:IncomeTax.Deduction.WorkRelatedOther.Amount

            RP:^IITR148 = tns:IncomeTax.Deduction.GiftDonation.Amount

            RP:^IITR502 = tns:IncomeTax.Deduction.TaxAffairManagementATOInterest.Amount

            RP:^IITR503 = tns:IncomeTax.Deduction.TaxAffairManagementLitigation.Amount

            RP:^IITR504 = tns:IncomeTax.Deduction.TaxAffairManagementOther.Amount

            RP:^IITR154 = tns:IncomeTax.LossesClaimedPrimaryProduction.Amount

            RP:^IITR156 = tns:IncomeTax.LossesClaimedNonPrimaryProduction.Amount

            RP:^IITR242 = tns:Income.PartnershipPrimaryProductionShare.Amount

            RP:^IITR243 = tns:Income.TrustPrimaryProductionNetShare.Amount

            RP.Prim:^IITR245 = tns:Expense.LandcareOperationsAcceleratedDepreciation.Amount

            RP:^IITR246 = tns:IncomeTax.Deduction.PartnershipsAndTrustPrimaryProductionShare.Amount

            RP:^IITR252 = tns:Income.PartnershipLessForeignIncomeNonPrimaryProductionShare.Amount

            RP:^IITR255 = tns:Income.TrustLessNetCapitalGainAndForeignIncomeNonPrimaryProductionShare.Amount

            RP:^IITR259 = tns:Income.TrustFrankedNonPrimaryProductionShare.Amount

            RP:^IITR260 = tns:Expense.LandcareOperationsNonPrimaryProduction.Amount

            RP:^IITR265 = tns:IncomeTax.Deduction.TrustPartnershipOtherNonPrimaryProductionNet.Amount

            RP:^IITR281 = tns:Income.PrimaryProductionNet.Amount

            RP:^IITR285 = tns:Income.NonPrimaryProductionNet.Amount

            RP:^IITR306 = tns:Income.FarmManagementDepositsOrRepaymentsNet.Amount

            RP:^IITR311 = tns:Capital.Gains.Net.Amount

            RP:^IITR316 = tns:Income.ControlledForeignCompaniesAttributableIncome.Amount

            RP:^IITR318 = tns:InternationalDealings.TransferorTrustAttributedForeignIncome.Amount

            RP:^IITR323 = tns:InternationalDealings.ForeignPensionOrAnnuityIncomeUndeductedPurchasePrice.Amount

            RP:^IITR327 = tns:Income.InternationalDealings.Net.Amount

            RP:^IITR334 = tns:Income.RealEstateProperty.Rental.Amount

            RP:^IITR335 = tns:IncomeTax.Deduction.RentalIncomeInterestDeductions.Amount

            RP:^IITR337 = tns:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount

            RP:^IITR339 = tns:IncomeTax.LifeInsuranceCompaniesAndFriendlySocietiesBonuses.Amount

            RP.{IncSeqNum}:^IITR348 = tns:IncomeTax.IncomeCategory1Total.Amount

            RP.{IncSeqNum}:^IITR561 = tns:IncomeTax.IncomeCategory2Other.Amount

            RP.{IncSeqNum}:^IITR566 = tns:IncomeTax.IncomeCategory3Total.Amount

            RP:^IITR358 = tns:IncomeTax.Deduction.ForeignPensionAnnuityIncomeUndeductedPurchasePrice.Amount

            RP.{SCSeqNum}:^IITR362 = tns:IncomeTax.Deduction.SuperannuationContribution.Amount

            RP:^IITR370 = tns:IncomeTax.Deduction.Election.Amount

            RP.{OTHSeqNum}:^IITR375 = tns:Expense.DeductibleOther.Amount
            */
            assertion = ((report.RPSaWSeqNumContextCollection != null ? report.RPSaWSeqNumContextCollection.Exists(context => context.IITR62 != null) == false : true) && report.IITR65 == null && (report.RPEmpLSPSeqNumContextCollection != null ? report.RPEmpLSPSeqNumContextCollection.Exists(context => context.IITR68 != null) == false && report.RPEmpLSPSeqNumContextCollection.Exists(context => context.IITR71 != null) == false : true) && report.IITR86 == null && report.IITR89 == null && report.IITR92 == null && report.IITR93 == null && report.IITR505 == null && report.IITR94 == null && report.IITR95 == null && report.IITR115 == null && report.IITR118 == null && report.IITR119 == null && report.IITR120 == null && report.IITR134 == null && report.IITR136 == null && report.IITR137 == null && report.IITR139 == null && report.IITR141 == null && report.IITR148 == null && report.IITR502 == null && report.IITR503 == null && report.IITR504 == null && report.IITR154 == null && report.IITR156 == null && report.IITR242 == null && report.IITR243 == null && report.IITR245 == null && report.IITR246 == null && report.IITR252 == null && report.IITR255 == null && report.IITR259 == null && report.IITR260 == null && report.IITR265 == null && report.IITR281 == null && report.IITR285 == null && report.IITR306 == null && report.IITR311 == null && report.IITR316 == null && report.IITR318 == null && report.IITR323 == null && report.IITR327 == null && report.IITR334 == null && report.IITR335 == null && report.IITR337 == null && report.IITR339 == null && (report.RPIncSeqNumContextCollection != null ? report.RPIncSeqNumContextCollection.Exists(context => context.IITR348 != null) == false && report.RPIncSeqNumContextCollection.Exists(context => context.IITR561 != null) == false && report.RPIncSeqNumContextCollection.Exists(context => context.IITR566 != null) == false : true) && report.IITR358 == null && (report.RPSCSeqNumContextCollection != null ? report.RPSCSeqNumContextCollection.Exists(context => context.IITR362 != null) == false : true) && report.IITR370 == null && (report.RPOTHSeqNumContextCollection != null ? report.RPOTHSeqNumContextCollection.Exists(context => context.IITR375 != null) == false : true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.300013",
                    Severity = ProcessMessageSeverity.Warning,
                    Description = @"No income or deduction amounts present. Is this a nil return?",
                    Location = "/xbrli:xbrl/tns:Income.SalaryOrWages.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300013" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR62", Value = "null" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR65", Value = report.IITR65.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR68", Value = "null" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR71", Value = "null" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR86", Value = report.IITR86.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR89", Value = report.IITR89.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR92", Value = report.IITR92.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR93", Value = report.IITR93.GetValueOrDefault().ToString() });
                
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR505", Value = report.IITR505.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR94", Value = report.IITR94.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR95", Value = report.IITR95.GetValueOrDefault().ToString() });
                
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR115", Value = report.IITR115.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR118", Value = report.IITR118.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR119", Value = report.IITR119.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR120", Value = report.IITR120.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR134", Value = report.IITR134.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR136", Value = report.IITR136.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR137", Value = report.IITR137.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR139", Value = report.IITR139.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR141", Value = report.IITR141.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR148", Value = report.IITR148.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR502", Value = report.IITR502.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR503", Value = report.IITR503.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR504", Value = report.IITR504.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR154", Value = report.IITR154.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR156", Value = report.IITR156.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR242", Value = report.IITR242.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR243", Value = report.IITR243.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR245", Value = report.IITR245.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR246", Value = report.IITR246.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR252", Value = report.IITR252.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR255", Value = report.IITR255.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR259", Value = report.IITR259.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR260", Value = report.IITR260.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR265", Value = report.IITR265.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR281", Value = report.IITR281.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR285", Value = report.IITR285.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR306", Value = report.IITR306.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR311", Value = report.IITR311.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR316", Value = report.IITR316.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR318", Value = report.IITR318.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR323", Value = report.IITR323.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR327", Value = report.IITR327.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR334", Value = report.IITR334.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR335", Value = report.IITR335.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR337", Value = report.IITR337.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR339", Value = report.IITR339.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR348", Value = "null" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR561", Value = "null" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR566", Value = "null" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR358", Value = report.IITR358.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR362", Value = "null" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR370", Value = report.IITR370.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR375", Value = "null" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.300013

            #region VR.ATO.IITR.300016

            /*  VR.ATO.IITR.300016
            Where deductions have been claimed for either; Work related car expenses, Work related travel expenses, Work related uniform, occupation specific or protective clothing, laundry and dry cleaning expenses, Work related self-education expenses, or Other work related expenses; without Salary and wage or Allowance income being declared, check Work related expense deductions as they are normally claimed only if Salary or Allowance income is declared.

            Legacy Rule Format:
            IF ([iitr65] = NULL) AND ALL OCCURRENCES OF([iitr62]) = NULL AND ([iitr134] > 0 OR [iitr136] > 0 OR [iitr137] > 0 OR [iitr139] > 0 OR [iitr141] > 0)
                 RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IITR65 = NULL) AND ^IITR62 = NULL AND (^IITR134 > 0 OR ^IITR136 > 0 OR ^IITR137 > 0 OR ^IITR139 > 0 OR ^IITR141 > 0)

            Data Elements:

            RP:^IITR65 = tns:Income.AllowancesEarningsTipsDirectorsFees.Amount

            RP.{SaWSeqNum}:^IITR62 = tns:Income.SalaryOrWages.Amount

            RP:^IITR134 = tns:IncomeTax.Deduction.Car.Amount

            RP:^IITR136 = tns:IncomeTax.Deduction.Travel.Amount

            RP:^IITR137 = tns:IncomeTax.Deduction.Clothing.Amount

            RP:^IITR139 = tns:IncomeTax.Deduction.EducationNet.Amount

            RP:^IITR141 = tns:IncomeTax.Deduction.WorkRelatedOther.Amount
            */
            assertion = (report.IITR65 == null && (report.RPSaWSeqNumContextCollection != null ? report.RPSaWSeqNumContextCollection.Exists(context => context.IITR62 != null) == false : true) && (report.IITR134.GetValueOrDefault() > 0 || report.IITR136.GetValueOrDefault() > 0 || report.IITR137.GetValueOrDefault() > 0 || report.IITR139.GetValueOrDefault() > 0 || report.IITR141.GetValueOrDefault() > 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.300016",
                    Severity = ProcessMessageSeverity.Warning,
                    Description = @"Work related expense deductions can only be claimed if salary or allowance income is declared",
                    Location = "/xbrli:xbrl/tns:Income.AllowancesEarningsTipsDirectorsFees.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300016" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR65", Value = report.IITR65.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR62", Value = "null" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR134", Value = report.IITR134.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR136", Value = report.IITR136.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR137", Value = report.IITR137.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR139", Value = report.IITR139.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR141", Value = report.IITR141.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.300016

            #region VR.ATO.IITR.300039

            /*  VR.ATO.IITR.300039
            A loss amount has been entered at primary production business income or non-primary production business income or Net Personal Services Income, but there are no Business loss activity details.

            Legacy Rule Format:
            IF ([iitr281] < 0 OR [iitr285] < 0 OR [iitr279] < 0) AND [iitr479] = NULLORBLANK
              RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IITR281 < 0 OR ^IITR285 < 0 OR ^IITR279 < 0) AND ^IITR479 = BLANK

            Data Elements:

            RP.{LOSSSeqNum}:^IITR479 = tns:OrganisationDetails.MainIncomeActivity.Description

            RP:^IITR281 = tns:Income.PrimaryProductionNet.Amount

            RP:^IITR285 = tns:Income.NonPrimaryProductionNet.Amount

            RP:^IITR279 = tns:Income.PersonalServicesIncome.Net.Amount
            */
            assertion = ((report.IITR281.GetValueOrDefault() < 0 || report.IITR285.GetValueOrDefault() < 0 || report.IITR279.GetValueOrDefault() < 0) && (report.RPLOSSSeqNumContextCollection != null ? report.RPLOSSSeqNumContextCollection.Exists(context => string.IsNullOrWhiteSpace(context.IITR479) == false) == false : true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.300039",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"'Business loss activity - Description of activity' is required",
                    LongDescription = @"When either a loss amount for primary production business income, non-primary production business income or Net Personal Services Income has been entered, Business loss activity details must be present",
                    Location = "/xbrli:xbrl/tns:OrganisationDetails.MainIncomeActivity.Description",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300039" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR281", Value = report.IITR281.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR285", Value = report.IITR285.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR279", Value = report.IITR279.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR479", Value = "null" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.300039

            #region VR.ATO.IITR.300087

            /*  VR.ATO.IITR.300087
            Taxpayer is ineligible to make deposits and claim deductions for deductible deposits because taxpayer's non-primary production income exceeded the threshold amount.

            Legacy Rule Format:
            IF ([iitr302] > 0) AND (([iitr157] - ([iitr242] + [iitr243] - [iitr245] - [iitr246] + [iitr281] + [iitr299] + [iitr306]) - Sum(ConditionalValue([iitr76] = SET("D","B","N"),[iitr75],0)) - Sum(ConditionalValue([iitr102] = FALSE,([iitr99] + [iitr100]),0)) - [iitr311] + [iitr154] + [iitr156]) > 100000)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IITR302 > 0) AND ((^IITR157 - (^IITR242 + ^IITR243 - ^IITR245 - ^IITR246 + ^IITR281 + ^IITR299 + ^IITR306) - Sum(ConditionalValue(InSet(^IITR76, '"D","B","N"'),^IITR75,0)) - Sum(ConditionalValue(^IITR102 = FALSE,(^IITR99 + ^IITR100),0)) - ^IITR311 + ^IITR154 + ^IITR156) > 100000)

            Data Elements:

            RP:^IITR302 = tns:IncomeTax.Deduction.FarmManagementDeposits.Amount

            RP:^IITR157 = tns:Income.Taxable.Amount

            RP:^IITR242 = tns:Income.PartnershipPrimaryProductionShare.Amount

            RP:^IITR243 = tns:Income.TrustPrimaryProductionNetShare.Amount

            RP.Prim:^IITR245 = tns:Expense.LandcareOperationsAcceleratedDepreciation.Amount

            RP:^IITR246 = tns:IncomeTax.Deduction.PartnershipsAndTrustPrimaryProductionShare.Amount

            RP:^IITR281 = tns:Income.PrimaryProductionNet.Amount

            RP:^IITR299 = tns:Tax.Losses.DeferredNonCommercialPrimaryProduction.Amount

            RP:^IITR306 = tns:Income.FarmManagementDepositsOrRepaymentsNet.Amount

            RP.{ETPSeqNum}:^IITR76 = tns:IncomeTax.PayAsYouGoWithholding.EmploymentTerminationPaymentType.Code

            RP.{ETPSeqNum}:^IITR75 = tns:Income.EmploymentTerminationPaymentTaxable.Amount

            RP.{SLSPSeqNum}:^IITR102 = tns:Income.Superannuation.DeathBenefitPaymentNonDependant.Indicator

            RP.{SLSPSeqNum}:^IITR99 = tns:Income.Superannuation.LumpSumPaymentTaxableComponentTaxedElement.Amount

            RP.{SLSPSeqNum}:^IITR100 = tns:Income.Superannuation.LumpSumPaymentTaxableComponentUntaxedElement.Amount

            RP:^IITR311 = tns:Capital.Gains.Net.Amount

            RP:^IITR154 = tns:IncomeTax.LossesClaimedPrimaryProduction.Amount

            RP:^IITR156 = tns:IncomeTax.LossesClaimedNonPrimaryProduction.Amount
            */
            assertion = (report.IITR302.GetValueOrDefault() > 0 && report.IITR157.GetValueOrDefault() - (report.IITR242.GetValueOrDefault() + report.IITR243.GetValueOrDefault() - report.IITR245.GetValueOrDefault() - report.IITR246.GetValueOrDefault() + report.IITR281.GetValueOrDefault() + report.IITR299.GetValueOrDefault() + report.IITR306.GetValueOrDefault()) - (report.RPETPSeqNumContextCollection != null ? report.RPETPSeqNumContextCollection.FindAll(context => IsMatch(context.IITR76, @"^(D|B|N)$")).Sum(f => f.IITR75.GetValueOrDefault()) : 0) - (report.RPSLSPSeqNumContextCollection != null ? report.RPSLSPSeqNumContextCollection.FindAll(context => context.IITR102 == false).Sum(f => f.IITR99.GetValueOrDefault() + f.IITR100.GetValueOrDefault()) : 0) - report.IITR311.GetValueOrDefault() + report.IITR154.GetValueOrDefault() + report.IITR156.GetValueOrDefault() > 100000);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.300087",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Taxpayer is ineligible to make deposits and claim deductions for deductible deposits",
                    LongDescription = @"Taxpayer is ineligible to make deposits and claim deductions for deductible deposits because taxpayer's non-primary production income exceeded the threshold amount",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.FarmManagementDeposits.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300087" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR302", Value = report.IITR302.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR157", Value = report.IITR157.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR242", Value = report.IITR242.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR243", Value = report.IITR243.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR245", Value = report.IITR245.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR246", Value = report.IITR246.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR281", Value = report.IITR281.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR299", Value = report.IITR299.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR306", Value = report.IITR306.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR75", Value = (report.RPETPSeqNumContextCollection != null ? report.RPETPSeqNumContextCollection.FindAll(context => IsMatch(context.IITR76, @"^(D|B|N)$")).Sum(f => f.IITR75).ToString() : "null") });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR99", Value = (report.RPSLSPSeqNumContextCollection != null ? report.RPSLSPSeqNumContextCollection.FindAll(context => context.IITR102 == false).Sum(f => f.IITR99).ToString() : "null") });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR100", Value = (report.RPSLSPSeqNumContextCollection != null ? report.RPSLSPSeqNumContextCollection.FindAll(context => context.IITR102 == false).Sum(f => f.IITR100).ToString() : "null") });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR311", Value = report.IITR311.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR154", Value = report.IITR154.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR156", Value = report.IITR156.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.300087

            #region VR.ATO.IITR.300094

            /*  VR.ATO.IITR.300094
            If the total of Net rental property income less Net rental property deductions is less than zero then the difference must be inserted at Net Rental property loss. If the difference exceeds zero then zero must be inserted at Net Rental property loss.
    
            Legacy Rule Format:
            IF ((([iitr250] + [iitr283] + [iitr292] + [iitr296] + [iitr324] + ([iitr334] - ([iitr335] + [iitr336] + [iitr337])) + SUM([iitr346])) < ([iitr143] + [iitr263] + SUM([iitr373])))
               AND ((([iitr205] = NULL)
                  OR ([iitr205] <> (([iitr143] + [iitr263] + SUM([iitr373])) - ([iitr250] + [iitr283] + [iitr292] + [iitr296] + [iitr324] + ([iitr334] - ([iitr335] + [iitr336] + [iitr337])) + SUM([iitr346])))))))
            OR ((([iitr250] + [iitr283] + [iitr292] + [iitr296] + [iitr324] + ([iitr334] - ([iitr335] + [iitr336] + [iitr337])) + SUM([iitr346])) >= ([iitr143] + [iitr263] + SUM([iitr373]))) 
               AND ([iitr205] <> 0))
                    RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (((^IITR250 + ^IITR283 + ^IITR292 + ^IITR296 + ^IITR324 + (^IITR334 - (^IITR335 + ^IITR336 + ^IITR337)) + Sum(^IITR346)) < (^IITR143 + ^IITR263 + Sum(^IITR373)))   AND (((^IITR205 <> NULL)      AND (^IITR205 <> ((^IITR143 + ^IITR263 + Sum(^IITR373)) - (^IITR250 + ^IITR283 + ^IITR292 + ^IITR296 + ^IITR324 + (^IITR334 - (^IITR335 + ^IITR336 + ^IITR337)) + Sum(^IITR346))))))) OR (((^IITR250 + ^IITR283 + ^IITR292 + ^IITR296 + ^IITR324 + (^IITR334 - (^IITR335 + ^IITR336 + ^IITR337)) + Sum(^IITR346)) >= (^IITR143 + ^IITR263 + Sum(^IITR373)))    AND (^IITR205 <> 0))
    
            Data Elements:
    
            ^IITR205 = RP:Tax.Losses.RentalPropertyNet.Amount
    
            ^IITR250 = RP:Income.PartnershipRentalNonPrimaryProductionShare.Amount
    
            ^IITR283 = RP:Income.RentalPropertyNonPrimaryProductionNet.Amount
    
            ^IITR292 = RP:Tax.Losses.DeferredNonCommercialPartnershipRentalShare.Amount
    
            ^IITR296 = RP:Tax.Losses.DeferredNonCommercialSoleTraderRental.Amount
    
            ^IITR324 = RP:InternationalDealings.ForeignRent.Amount
    
            ^IITR334 = RP:Income.RealEstateProperty.Rental.Amount
    
            ^IITR335 = RP:IncomeTax.Deduction.RentalIncomeInterestDeductions.Amount
    
            ^IITR336 = RP:IncomeTax.Deduction.CapitalWorksDeductions.Amount
    
            ^IITR337 = RP:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
    
            ^IITR346 = RP.{IncSeqNum}:IncomeTax.AssessableLowValuePoolDeductionRentalPropertyOther.Amount
    
            ^IITR143 = RP:IncomeTax.Deduction.LowValuePoolRentalProperties.Amount
    
            ^IITR263 = RP:IncomeTax.Deduction.PartnershipRentalNonPrimaryProduction.Amount
    
            ^IITR373 = RP.{OTHSeqNum}:IncomeTax.Deduction.ForeignRentalIncomeOther.Amount
            */
            assertion = (((report.IITR250.GetValueOrDefault() + report.IITR283.GetValueOrDefault() + report.IITR292.GetValueOrDefault() + report.IITR296.GetValueOrDefault() + report.IITR324.GetValueOrDefault() + (report.IITR334.GetValueOrDefault() - (report.IITR335.GetValueOrDefault() + report.IITR336.GetValueOrDefault() + report.IITR337.GetValueOrDefault())) + (report.RPIncSeqNumContextCollection == null ? 0 : report.RPIncSeqNumContextCollection.Sum(f => f.IITR346.GetValueOrDefault())) < report.IITR143.GetValueOrDefault() + report.IITR263.GetValueOrDefault() + (report.RPOTHSeqNumContextCollection == null ? 0 : report.RPOTHSeqNumContextCollection.Sum(f => f.IITR373.GetValueOrDefault()))) && (report.IITR205 == null || report.IITR205.GetValueOrDefault() != report.IITR143.GetValueOrDefault() + report.IITR263.GetValueOrDefault() + (report.RPOTHSeqNumContextCollection == null ? 0 : report.RPOTHSeqNumContextCollection.Sum(f => f.IITR373.GetValueOrDefault())) - (report.IITR250.GetValueOrDefault() + report.IITR283.GetValueOrDefault() + report.IITR292.GetValueOrDefault() + report.IITR296.GetValueOrDefault() + report.IITR324.GetValueOrDefault() + (report.IITR334.GetValueOrDefault() - (report.IITR335.GetValueOrDefault() + report.IITR336.GetValueOrDefault() + report.IITR337.GetValueOrDefault())) + (report.RPIncSeqNumContextCollection == null ? 0 : report.RPIncSeqNumContextCollection.Sum(f => f.IITR346.GetValueOrDefault()))))) || (report.IITR250.GetValueOrDefault() + report.IITR283.GetValueOrDefault() + report.IITR292.GetValueOrDefault() + report.IITR296.GetValueOrDefault() + report.IITR324.GetValueOrDefault() + (report.IITR334.GetValueOrDefault() - (report.IITR335.GetValueOrDefault() + report.IITR336.GetValueOrDefault() + report.IITR337.GetValueOrDefault())) + (report.RPIncSeqNumContextCollection == null ? 0 : report.RPIncSeqNumContextCollection.Sum(f => f.IITR346.GetValueOrDefault())) >= report.IITR143.GetValueOrDefault() + report.IITR263.GetValueOrDefault() + (report.RPOTHSeqNumContextCollection == null ? 0 : report.RPOTHSeqNumContextCollection.Sum(f => f.IITR373.GetValueOrDefault())) && report.IITR205.GetValueOrDefault() != 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.300094",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net Rental Property Loss is incorrect",
                    LongDescription = @"If the total of Net rental property income less Net rental property deductions is less than zero then the difference must be inserted at Net Rental property loss. If the difference exceeds zero then zero must be inserted at Net Rental property loss.",
                    Location = "/xbrli:xbrl/tns:Tax.Losses.RentalPropertyNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300094" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR250", Value = report.IITR250.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR283", Value = report.IITR283.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR292", Value = report.IITR292.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR296", Value = report.IITR296.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR324", Value = report.IITR324.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR334", Value = report.IITR334.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR335", Value = report.IITR335.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR336", Value = report.IITR336.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR337", Value = report.IITR337.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR346", Value = "IITR346" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR143", Value = report.IITR143.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR263", Value = report.IITR263.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR373", Value = "IITR373" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR205", Value = report.IITR205.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.300094

            #region VR.ATO.IITR.310025

            /*  VR.ATO.IITR.310025
            For Business loss activity only activity 1 has been entered and the Type of loss code for Business loss activity 1 is 8 . The Net loss amount does not equal the sum of primary production deferred losses and non-primary production deferred losses.

            Legacy Rule Format:
            IF ([iitr479:2] = NULLORBLANK  AND [iitr479:3] = NULLORBLANK AND [iitr482:1] = 8 AND ((([iitr487:1] = NULL) AND (([iitr299] + [iitr300]) <> 0)) OR (([iitr487:1] <> NULL) AND ([iitr487:1] <> ([iitr299] + [iitr300])))))
               RETURN VALIDATION MESSAGE
            ENDIF

            [iitr479:2] = ([iitr479] WHERE RP.{LOSSSeqNum}:LossesSequenceContainer = 2)
            [iitr479:3] = ([iitr479] WHERE RP.{LOSSSeqNum}:LossesSequenceContainer = 3)
            [iitr482:1] = ([iitr482] WHERE RP.{LOSSSeqNum}:LossesSequenceContainer = 1)
            [iitr487:1] = ([iitr487] WHERE RP.{LOSSSeqNum}:LossesSequenceContainer = 1)

            Technical Business Rule Format:
            ([iitr479:2] = BLANK  AND [iitr479:3] = BLANK AND [iitr482:1] = 8 AND ((([iitr487:1] = NULL) AND ((^IITR299 + ^IITR300) <> 0)) OR (([iitr487:1] <> NULL) AND ([iitr487:1] <> (^IITR299 + ^IITR300)))))           [iitr479:2] = (^IITR479 WHERE RP.{LOSSSeqNum}:LossesSequenceContainer = 2) [iitr479:3] = (^IITR479 WHERE RP.{LOSSSeqNum}:LossesSequenceContainer = 3) [iitr482:1] = (^IITR482 WHERE RP.{LOSSSeqNum}:LossesSequenceContainer = 1) [iitr487:1] = (^IITR487 WHERE RP.{LOSSSeqNum}:LossesSequenceContainer = 1)

            Data Elements:

            RP.{LOSSSeqNum}:^IITR479 = tns:OrganisationDetails.MainIncomeActivity.Description

            RP:^IITR299 = tns:Tax.Losses.DeferredNonCommercialPrimaryProduction.Amount

            RP:^IITR300 = tns:Tax.Losses.DeferredNonCommercialNonPrimaryProduction.Amount

            RP.{LOSSSeqNum}:^IITR482 = tns:IncomeTax.LossesType.Code

            RP.{LOSSSeqNum}:^IITR487 = tns:Tax.Losses.Net.Amount
            */
            assertion = (report.RPLOSSSeqNumContextCollection != null && report.RPLOSSSeqNumContextCollection.Exists(f => f.LossesSequenceDimensionValue == "1") == true && report.RPLOSSSeqNumContextCollection.Exists(f => f.LossesSequenceDimensionValue == "2") == true && report.RPLOSSSeqNumContextCollection.Exists(f => f.LossesSequenceDimensionValue == "3") == true && string.IsNullOrWhiteSpace(report.RPLOSSSeqNumContextCollection.Find(f => f.LossesSequenceDimensionValue == "2").IITR479) && string.IsNullOrWhiteSpace(report.RPLOSSSeqNumContextCollection.Find(f => f.LossesSequenceDimensionValue == "3").IITR479) && report.RPLOSSSeqNumContextCollection.Find(f => f.LossesSequenceDimensionValue == "1").IITR482 == "8" && ((report.RPLOSSSeqNumContextCollection.Find(f => f.LossesSequenceDimensionValue == "1").IITR487 == null && report.IITR299.GetValueOrDefault() + report.IITR300.GetValueOrDefault() != 0) || (report.RPLOSSSeqNumContextCollection.Find(f => f.LossesSequenceDimensionValue == "1").IITR487 != null && report.IITR299.GetValueOrDefault() + report.IITR300.GetValueOrDefault() != report.RPLOSSSeqNumContextCollection.Find(f => f.LossesSequenceDimensionValue == "1").IITR487)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.310025",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Primary production and/or Non-primary production deferred losses amount incorrect",
                    LongDescription = @"For Business loss activity, only activity 1 has been entered and the 'Type of loss code' for Business loss activity 1 is '8' . The Net loss amount must equal the sum of primary production deferred losses and non-primary production deferred losses.",
                    Location = "/xbrli:xbrl/tns:OrganisationDetails.MainIncomeActivity.Description[@contextRef='" + report.RPLOSSSeqNumContextCollection.Find(f => f.LossesSequenceDimensionValue == "2").Id + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310025" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR299", Value = report.IITR299.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR300", Value = report.IITR300.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR479:2", Value = (report.RPLOSSSeqNumContextCollection != null ? report.RPLOSSSeqNumContextCollection.Find(f => f.LossesSequenceDimensionValue == "2").IITR479 : "null") });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR479:3", Value = (report.RPLOSSSeqNumContextCollection != null ? report.RPLOSSSeqNumContextCollection.Find(f => f.LossesSequenceDimensionValue == "3").IITR479 : "null") });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR482", Value = (report.RPLOSSSeqNumContextCollection != null ? report.RPLOSSSeqNumContextCollection.Find(f => f.LossesSequenceDimensionValue == "1").IITR482 : "null") });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR487", Value = (report.RPLOSSSeqNumContextCollection != null ? report.RPLOSSSeqNumContextCollection.Find(f => f.LossesSequenceDimensionValue == "1").IITR487.GetValueOrDefault().ToString() : "null") });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.310025

            #region VR.ATO.IITR.410010

            /*  VR.ATO.IITR.410010
            For Business loss activity details, If Business loss activity 2 has been completed and business loss activity 1 is blank, or if business loss activity 3 has been completed and business loss activity 2 is blank, it has been entered incorrectly as these details must be entered in the correct sequence.
    
            Legacy Rule Format:
            IF ([iitr479:2] <> NULL AND [iitr479:1] = NULL) OR ([iitr479:3] <> NULL AND [iitr479:2] = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            
            [iitr479:1] = ([iitr479] WHERE RP.{LOSSSeqNum}:LossesSequenceContainer = 1)
            [iitr479:2] = ([iitr479] WHERE RP.{LOSSSeqNum}:LossesSequenceContainer = 2)
            [iitr479:3] = ([iitr479] WHERE RP.{LOSSSeqNum}:LossesSequenceContainer = 3)

            Technical Business Rule Format:
            ([iitr479:2] <> NULL AND [iitr479:1] = NULL) OR ([iitr479:3] <> NULL AND [iitr479:2] = NULL)            [iitr479:1] = (^IITR479 WHERE RP.{LOSSSeqNum}:LossesSequenceContainer = 1) [iitr479:2] = (^IITR479 WHERE RP.{LOSSSeqNum}:LossesSequenceContainer = 2) [iitr479:3] = (^IITR479 WHERE RP.{LOSSSeqNum}:LossesSequenceContainer = 3)
    
            Data Elements:
    
            RP.{LOSSSeqNum}:^IITR479 = tns:OrganisationDetails.MainIncomeActivity.Description
            */
            IITR2018.RPLOSSSeqNumContext lossContext1 = null;
            IITR2018.RPLOSSSeqNumContext lossContext2 = null;
            IITR2018.RPLOSSSeqNumContext lossContext3 = null;

            // Find all of the LOSSSeqNum contexts
            if (report.RPLOSSSeqNumContextCollection != null)
            {
                lossContext1 = report.RPLOSSSeqNumContextCollection.Find(f => f.LossesSequenceDimensionValue == "1");
                lossContext2 = report.RPLOSSSeqNumContextCollection.Find(f => f.LossesSequenceDimensionValue == "2");
                lossContext3 = report.RPLOSSSeqNumContextCollection.Find(f => f.LossesSequenceDimensionValue == "3");
            }

            assertion = (((lossContext2 != null && lossContext2.IITR479 != null) && (lossContext1 == null || lossContext1.IITR479 == null))
                      || ((lossContext3 != null && lossContext3.IITR479 != null) && (lossContext2 == null || lossContext2.IITR479 == null)));

            if (assertion)
            {
                // Find the first context where IITR479 is null, or if that doesn't exist take the first context in the collection as the location path
                string locationPath = report.RPLOSSSeqNumContextCollection.Exists(f => f.IITR479 == null) ? report.RPLOSSSeqNumContextCollection.First(f => f.IITR479 == null).Id : report.RPLOSSSeqNumContextCollection[0].Id;

                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.410010",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Business loss activity details out of sequence",
                    LongDescription = @"For Business loss activity details, If Business loss activity 2 has been completed and business loss activity 1 is blank, or if business loss activity 3 has been completed and business loss activity 2 is blank, it has been entered incorrectly as these details must be entered in the correct sequence",
                    Location = "/xbrli:xbrl/tns:OrganisationDetails.MainIncomeActivity.Description[@contextRef='" + locationPath + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.410010" } },
                };

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR479:1", Value = (report.RPLOSSSeqNumContextCollection != null && lossContext1 != null && lossContext1.IITR479 != null ? lossContext1.IITR479 : "null") });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR479:2", Value = (report.RPLOSSSeqNumContextCollection != null && lossContext2 != null && lossContext2.IITR479 != null ? lossContext2.IITR479 : "null") });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR479:3", Value = (report.RPLOSSSeqNumContextCollection != null && lossContext3 != null && lossContext3.IITR479 != null ? lossContext3.IITR479 : "null") });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.410010

            #region VR.ATO.IITR.410012

            /*  VR.ATO.IITR.410012
            When the loss type from business loss activity details is '8' (loss is to be deferred) for all 3 activities, the sum of primary production deferred losses and non-primary production deferred losses must not be less than the sum of net loss amounts.

            Legacy Rule Format:
            IF ([iitr482:1] = 8) AND ([iitr482:2] = 8) AND ([iitr482:3] = 8) AND (SUM([iitr487]) > ([iitr299] + [iitr300]))
               RETURN VALIDATION MESSAGE
            ENDIF

            [iitr482:1] = ([iitr482] WHERE RP.{LOSSSeqNum}:LossesSequenceContainer = 1)
            [iitr482:2] = ([iitr482] WHERE RP.{LOSSSeqNum}:LossesSequenceContainer = 2)
            [iitr482:3] = ([iitr482] WHERE RP.{LOSSSeqNum}:LossesSequenceContainer = 3)

            Technical Business Rule Format:
            ([iitr482:1] = 8) AND ([iitr482:2] = 8) AND ([iitr482:3] = 8) AND (Sum(^IITR487) > (^IITR299 + ^IITR300))           [iitr482:1] = (^IITR482 WHERE RP.{LOSSSeqNum}:LossesSequenceContainer = 1) [iitr482:2] = (^IITR482 WHERE RP.{LOSSSeqNum}:LossesSequenceContainer = 2) [iitr482:3] = (^IITR482 WHERE RP.{LOSSSeqNum}:LossesSequenceContainer = 3)

            Data Elements:

            RP.{LOSSSeqNum}:^IITR482 = tns:IncomeTax.LossesType.Code

            RP.{LOSSSeqNum}:^IITR487 = tns:Tax.Losses.Net.Amount

            RP:^IITR299 = tns:Tax.Losses.DeferredNonCommercialPrimaryProduction.Amount

            RP:^IITR300 = tns:Tax.Losses.DeferredNonCommercialNonPrimaryProduction.Amount
            */
            assertion = (report.RPLOSSSeqNumContextCollection != null && report.RPLOSSSeqNumContextCollection.Find(f => f.LossesSequenceDimensionValue == "1") != null && report.RPLOSSSeqNumContextCollection.Find(f => f.LossesSequenceDimensionValue == "2") != null && report.RPLOSSSeqNumContextCollection.Find(f => f.LossesSequenceDimensionValue == "3") != null && report.RPLOSSSeqNumContextCollection.Find(f => f.LossesSequenceDimensionValue == "1").IITR482 == "8" && report.RPLOSSSeqNumContextCollection.Find(f => f.LossesSequenceDimensionValue == "2").IITR482 == "8" && report.RPLOSSSeqNumContextCollection.Find(f => f.LossesSequenceDimensionValue == "3").IITR482 == "8" && report.RPLOSSSeqNumContextCollection.Sum(f => f.IITR487.GetValueOrDefault()) > (report.IITR299.GetValueOrDefault() + report.IITR300.GetValueOrDefault()));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.410012",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Total Net loss cannot exceed the sum of Total primary production and non-primary production deferred losses",
                    LongDescription = @"When the loss type from business loss activity details is '8' (loss is to be deferred) for all 3 activities, the sum of primary production deferred losses and non-primary production deferred losses must not be less than the sum of net loss amounts",
                    Location = "/xbrli:xbrl/tns:IncomeTax.LossesType.Code[@contextRef='" + report.RPLOSSSeqNumContextCollection.Find(f => f.LossesSequenceDimensionValue == "1").Id + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.410012" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR299", Value = report.IITR299.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR300", Value = report.IITR300.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR482:1", Value = (report.RPLOSSSeqNumContextCollection != null ? report.RPLOSSSeqNumContextCollection.Find(f => f.LossesSequenceDimensionValue == "1").IITR482 : "null") });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR482:2", Value = (report.RPLOSSSeqNumContextCollection != null ? report.RPLOSSSeqNumContextCollection.Find(f => f.LossesSequenceDimensionValue == "2").IITR482 : "null") });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR482:3", Value = (report.RPLOSSSeqNumContextCollection != null ? report.RPLOSSSeqNumContextCollection.Find(f => f.LossesSequenceDimensionValue == "3").IITR482 : "null") });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR487", Value = report.RPLOSSSeqNumContextCollection.Sum(f => f.IITR487.GetValueOrDefault()).ToString() });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.410012

            #region VR.ATO.IITR.410018

            /*  VR.ATO.IITR.410018
            Primary production deferred losses and/or non-primary production deferred losses have been entered without Business loss activity details being completed.

            Legacy Rule Format:
            IF ([iitr299] + [iitr300]) > 0 AND (COUNT(RP.{LOSSSeqNum}) = 0 OR ALL OCCURRENCES OF ([iitr479]) = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^IITR299 + ^IITR300) > 0 AND (Count(^Context) = 0 OR  ^IITR479 = BLANK)

            Data Elements:

            RP.{LOSSSeqNum}:^IITR479 = tns:OrganisationDetails.MainIncomeActivity.Description

            RP:^IITR299 = tns:Tax.Losses.DeferredNonCommercialPrimaryProduction.Amount

            RP:^IITR300 = tns:Tax.Losses.DeferredNonCommercialNonPrimaryProduction.Amount

            RP:^Context = RP:RP.{LOSSSeqNum}
            */
            assertion = (report.IITR299.GetValueOrDefault() + report.IITR300.GetValueOrDefault() > 0 && (report.RPLOSSSeqNumContextCollection != null ? Count(report.RPLOSSSeqNumContextCollection) == 0 || report.RPLOSSSeqNumContextCollection.Exists(context => string.IsNullOrWhiteSpace(context.IITR479) == false) == false : true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.410018",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Business loss activity details is required",
                    LongDescription = @"Primary production deferred losses and/or non-primary production deferred losses have been entered without Business loss activity details being completed",
                    Location = "/xbrli:xbrl/tns:OrganisationDetails.MainIncomeActivity.Description",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.410018" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR299", Value = report.IITR299.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR300", Value = report.IITR300.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR479", Value = "null" });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.410018

            #region VR.ATO.IITR.410034

            /*  VR.ATO.IITR.410034
            For income test purposes, spouse's adjusted taxable income (ATI) is the sum of spouse's taxable income, spouse's adjusted fringe benefit (Your spouse's total reportable fringe benefits amount from employers exempt under section 57A x 0.530 plus Your spouse's total reportable fringe benefits amounts from employers not exempt under section 57A), spouse's target foreign income, spouse's tax-free pensions (exempt pensions plus tax-free pensions paid under Military Rehabilitation and Compensation Act), spouse's total net investment loss, spouse's reportable superannuation contributions less child support the spouse paid. For the benefit of other calculations, ensure the ATI amount is correct.
    
            Legacy Rule Format:
            IF [iitr223] <> TRUE AND (([iitr237] = NULL AND (([iitr225] + ConditionalValue(([iitr492] + [iitr493]) > 3772, (([iitr492] * 0.530) + [iitr493]), 0) + [iitr233] + [iitr230] + [iitr232] + [iitr234] + [iitr231] - [iitr235]) +/- 1) <> 0) OR ([iitr237] <> NULL AND [iitr237] <> (([iitr225] + ConditionalValue(([iitr492] + [iitr493]) > 3772, (([iitr492] * 0.530) + [iitr493]), 0) + [iitr233] + [iitr230] + [iitr232] + [iitr234] + [iitr231] - [iitr235]) +/- 1)))
                RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IITR223 <> TRUE AND ((^IITR237 = NULL AND ((^IITR225 + ConditionalValue((^IITR492 + ^IITR493) > 3772, ((^IITR492 + ^IITR493) * 0.530), 0) + ^IITR233 + ^IITR230 + ^IITR232 + ^IITR234 + ^IITR231 - ^IITR235) +/- 1) <> 0) OR (^IITR237 <> NULL AND ^IITR237 <> ((^IITR225 + ConditionalValue((^IITR492 + ^IITR493) > 3772, ((^IITR492 + ^IITR493) * 0.530), 0) + ^IITR233 + ^IITR230 + ^IITR232 + ^IITR234 + ^IITR231 - ^IITR235) +/- 1)))
    
            Data Elements:
    
            RP.SPOUSE:^IITR223 = tns:PersonDemographicDetails.Deceased.Indicator
    
            RP.SPOUSE:^IITR237 = tns:Income.TaxableAdjusted.Amount
    
            RP.SPOUSE:^IITR225 = tns:Income.Taxable.Amount
    
            RP.SPOUSE.Eligible:^IITR492 = tns:Income.FringeBenefitsReportable.Amount
    
            RP.SPOUSE.Ineligible:^IITR493 = tns:Income.FringeBenefitsReportable.Amount
    
            RP.SPOUSE:^IITR233 = tns:InternationalDealings.ForeignIncomeTarget.Amount
    
            RP.SPOUSE:^IITR230 = tns:Pension.SpouseExempt.Amount
    
            RP.SPOUSE:^IITR232 = tns:Pension.TaxFree.Amount
    
            RP.SPOUSE:^IITR234 = tns:Tax.Losses.InvestmentNet.Amount
    
            RP.SPOUSE:^IITR231 = tns:SuperannuationContribution.EmployerReportable.Amount
    
            RP.SPOUSE:^IITR235 = tns:ChildSupport.Payment.Amount
            */
            
            assertion = (report.IITR223 != true && (report.IITR237 == null && OutsideRange(0, report.IITR225.GetValueOrDefault() + ConditionalValue(report.IITR492.GetValueOrDefault() + report.IITR493.GetValueOrDefault() > 3772, (report.IITR492.GetValueOrDefault() * 0.530M) + report.IITR493.GetValueOrDefault(), 0) + report.IITR233.GetValueOrDefault() + report.IITR230.GetValueOrDefault() + report.IITR232.GetValueOrDefault() + report.IITR234.GetValueOrDefault() + report.IITR231.GetValueOrDefault() - report.IITR235.GetValueOrDefault(), 1) || report.IITR237 != null && OutsideRange(report.IITR237.GetValueOrDefault(), report.IITR225.GetValueOrDefault() + ConditionalValue(report.IITR492.GetValueOrDefault() + report.IITR493.GetValueOrDefault() > 3772, (report.IITR492.GetValueOrDefault() * 0.530M) + report.IITR493.GetValueOrDefault(), 0) + report.IITR233.GetValueOrDefault() + report.IITR230.GetValueOrDefault() + report.IITR232.GetValueOrDefault() + report.IITR234.GetValueOrDefault() + report.IITR231.GetValueOrDefault() - report.IITR235.GetValueOrDefault(), 1)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.022013",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Spouse's ATI amount for Income test purposes is incorrect",
                    LongDescription = @"For income test purposes, spouse's adjusted taxable income (ATI) is the sum of spouse's taxable income, spouse's adjusted fringe benefit (Your spouse's total reportable fringe benefits amount from employers exempt under section 57A x 0.530 plus Your spouse's total reportable fringe benefits amounts from employers not exempt under section 57A), spouse's target foreign income, spouse's tax-free pensions (exempt pensions plus tax-free pensions paid under Military Rehabilitation and Compensation Act), spouse's total net investment loss, spouse's reportable superannuation contributions less child support the spouse paid. For the benefit of other calculations, ensure the ATI amount is correct.",
                    Location = "/xbrli:xbrl/tns:PersonDemographicDetails.Deceased.Indicator[@contextRef='" + report.RPSPOUSEId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.410034" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR223", Value = report.IITR223.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR225", Value = report.IITR225.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR230", Value = report.IITR230.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR231", Value = report.IITR231.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR232", Value = report.IITR232.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR233", Value = report.IITR233.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR234", Value = report.IITR234.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR235", Value = report.IITR235.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR237", Value = report.IITR237.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR492", Value = report.IITR492.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR493", Value = report.IITR493.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.410034

            #region VR.ATO.IITR.600097

            /*  VR.ATO.IITR.600097
            When a Working holiday maker net income amount above zero is provided, at least one of the following income amounts must be provided: a salary and wage associated with working holiday maker income; Share of net income from trusts less capital gains, foreign income and franked distributions; Net non-Primary Production income/loss; or Net small business income

            Legacy Rule Format:
            IF ([iitr499] > 0 AND ALL OCCURRENCES OF([iitr498]) = NULL AND [iitr255] = NULL AND [iitr285] = NULL AND [iitr557] = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^IITR499 > 0 AND ^IITR498 = NULL AND ^IITR255 = NULL AND ^IITR285 = NULL AND ^IITR557 = NULL

            Data Elements:

            RP.WHM:^IITR499 = tns:Income.Net.Amount

            RP.{SaWSeqNum}:^IITR498 = tns:Remuneration.WithholdingPaymentIncomeType.Code

            RP:^IITR255 = tns:Income.TrustLessNetCapitalGainAndForeignIncomeNonPrimaryProductionShare.Amount

            RP:^IITR285 = tns:Income.NonPrimaryProductionNet.Amount

            RP:^IITR557 = tns:Income.SmallBusinessEntityNet.Amount
            */
            assertion = (report.IITR499.GetValueOrDefault() > 0 && (report.RPSaWSeqNumContextCollection != null ? report.RPSaWSeqNumContextCollection.Exists(context => context.IITR498 != null) == false : true) && report.IITR255 == null && report.IITR285 == null && report.IITR557 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.600097",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Working holiday maker income amount details are incomplete",
                    LongDescription = @"When a Working holiday maker net income amount above zero is provided, at least one of the following income amounts must be provided; a salary and wage associated with working holiday maker income; Share of net income from trusts less capital gains, foreign income and franked distributions; Net non-Primary Production income/loss; or Net small business income",
                    Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.RPWHMId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.600097" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR499", Value = report.IITR499.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR498", Value = "null" });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR255", Value = report.IITR255.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR285", Value = report.IITR285.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR557", Value = report.IITR557.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.600097

            #region VR.ATO.IITR.600103

            /*  VR.ATO.IITR.600103
            Working holiday maker net income amount must not be less than the sum of working holiday maker Gross income less standard deductions

            Legacy Rule Format:
            IF [iitr499] <> NULL AND ANY OCCURRENCE OF([iitr498]) = "009" AND [iitr499] < (SUM(ConditionalValue([iitr498] = "009", [iitr62], 0)) - ([iitr134] + [iitr136] + [iitr137] + [iitr139] + [iitr141] + [iitr145] + [iitr146] + [iitr147] + [iitr148] + [iitr502] + [iitr503] + [iitr504]))

            Technical Business Rule Format:
            ^IITR499 <> NULL AND AnyOccurrence(^IITR498, ^IITR498 = '009') AND ^IITR499 < (Sum(ConditionalValue(^IITR498 = '009', ^IITR62, 0)) - (^IITR134 + ^IITR136 + ^IITR137 + ^IITR139 + ^IITR141 + ^IITR145 + ^IITR146 + ^IITR147 + ^IITR148 + ^IITR502 + ^IITR503 + ^IITR504))

            Data Elements:

            RP.WHM:^IITR499 = tns:Income.Net.Amount

            RP.{SaWSeqNum}:^IITR498 = tns:Remuneration.WithholdingPaymentIncomeType.Code

            RP.{SaWSeqNum}:^IITR62 = tns:Income.SalaryOrWages.Amount

            RP:^IITR134 = tns:IncomeTax.Deduction.Car.Amount

            RP:^IITR136 = tns:IncomeTax.Deduction.Travel.Amount

            RP:^IITR137 = tns:IncomeTax.Deduction.Clothing.Amount

            RP:^IITR139 = tns:IncomeTax.Deduction.EducationNet.Amount

            RP:^IITR141 = tns:IncomeTax.Deduction.WorkRelatedOther.Amount

            RP:^IITR145 = tns:IncomeTax.Deduction.LowValuePool.Amount

            RP:^IITR146 = tns:IncomeTax.Deduction.Interest.Amount

            RP:^IITR147 = tns:IncomeTax.Deduction.Dividend.Amount

            RP:^IITR148 = tns:IncomeTax.Deduction.GiftDonation.Amount

            RP:^IITR502 = tns:IncomeTax.Deduction.TaxAffairManagementATOInterest.Amount

            RP:^IITR503 = tns:IncomeTax.Deduction.TaxAffairManagementLitigation.Amount

            RP:^IITR504 = tns:IncomeTax.Deduction.TaxAffairManagementOther.Amount
            */
            assertion = (report.IITR499 != null && (report.RPSaWSeqNumContextCollection == null ? false : report.RPSaWSeqNumContextCollection.Any(IITR498Repeat => IITR498Repeat.IITR498 == @"009")) && (report.IITR499.GetValueOrDefault() < ConditionalValue(report.RPSaWSeqNumContextCollection.Any(IITR498Repeat => IITR498Repeat.IITR498 == @"009"), report.RPSaWSeqNumContextCollection.FindAll(IITR498Repeat => IITR498Repeat.IITR498 == @"009").Sum(IITR62Repeat => IITR62Repeat.IITR62.GetValueOrDefault()), 0) - (report.IITR134.GetValueOrDefault() + report.IITR136.GetValueOrDefault() + report.IITR137.GetValueOrDefault() + report.IITR139.GetValueOrDefault() + report.IITR141.GetValueOrDefault() + report.IITR145.GetValueOrDefault() + report.IITR146.GetValueOrDefault() + report.IITR147.GetValueOrDefault() + report.IITR148.GetValueOrDefault() + report.IITR502.GetValueOrDefault() + report.IITR503.GetValueOrDefault() + report.IITR504.GetValueOrDefault())));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.600103",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Working holiday maker net income amount is incorrect",
                    LongDescription = @"Working holiday maker net income amount must not be less than the sum of working holiday maker Gross income less standard deductions",
                    Location = "/xbrli:xbrl/tns:Income.Net.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.600103" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR499", Value = report.IITR499.GetValueOrDefault().ToString() });
                
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR62", Value = report.RPSaWSeqNumContextCollection.FindAll(IITR498Repeat => IITR498Repeat.IITR498 == @"009").Sum(IITR62Repeat => IITR62Repeat.IITR62).GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR134", Value = report.IITR134.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR136", Value = report.IITR136.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR137", Value = report.IITR137.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR139", Value = report.IITR139.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR141", Value = report.IITR141.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR145", Value = report.IITR145.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR146", Value = report.IITR146.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR147", Value = report.IITR147.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR148", Value = report.IITR148.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR502", Value = report.IITR502.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR503", Value = report.IITR503.GetValueOrDefault().ToString() });

                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR504", Value = report.IITR504.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.600103

            #region Repeating RPETPSeqNumContext

            if (report.RPETPSeqNumContextCollection != null)
            {
                foreach (IITR2018.RPETPSeqNumContext rpETPSeqNumContext in report.RPETPSeqNumContextCollection)
                {
                    #region VR.ATO.IITR.630009

                    /*  VR.ATO.IITR.630009
                    Where the total ETP payments for Death benefit ETP paid to a non-dependant of the deceased, from the same payer ABN, is greater than $200,000 (low CAP rate), and total tax withheld from ETP payments for Death benefit ETP paid to a non-dependant of the deceased from the same payer ABN is less than ETP taxable component * 32% (30% is etp2 + Medicare 2%), check ETP Tax withheld amount

                    Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{ETPSeqNum})
                    IF ([iitr76] = "N") AND ((([iitr546] <> NULL) AND (SUM(ConditionalValue([iitr546] = [iitr546:N],[iitr75],0)) > 200000) AND (SUM(ConditionalValue([iitr546] = [iitr546:N],[iitr74],0)) < (SUM(ConditionalValue([iitr546] = [iitr546:N],[iitr75],0)) * 0.32))) OR (([iitr77] <> NULL) AND (SUM(ConditionalValue([iitr77] = [iitr77:N],[iitr75],0)) > 200000) AND (SUM(ConditionalValue([iitr77] = [iitr77:N],[iitr74],0)) < (SUM(ConditionalValue([iitr77] = [iitr77:N],[iitr75],0)) * 0.32))))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    [iitr546:N] = ANY OTHER OCCURRENCE OF ConditionalValue(([iitr76] = "N"),([iitr546]),NULL)
                    [iitr77:N] = ANY OTHER OCCURRENCE OF ConditionalValue(([iitr76] = "N"),([iitr77]),NULL)

                    Technical Business Rule Format:
                    (^IITR76 = 'N') AND (((^IITR546 <> NULL) AND (Sum(ConditionalValue(^IITR546 = [iitr546:N],^IITR75,0)) > 200000) AND (Sum(ConditionalValue(^IITR546 = [iitr546:N],^IITR74,0)) < (Sum(ConditionalValue(^IITR546 = [iitr546:N],^IITR75,0)) * 0.32))) OR ((^IITR77 <> NULL) AND (Sum(ConditionalValue(^IITR77 = [iitr77:N],^IITR75,0)) > 200000) AND (Sum(ConditionalValue(^IITR77 = [iitr77:N],^IITR74,0)) < (Sum(ConditionalValue(^IITR77 = [iitr77:N],^IITR75,0)) * 0.32))))          [iitr546:N] = ANY OTHER OCCURRENCE OF ConditionalValue((^IITR76 = 'N'),^IITR546,NULL) [iitr77:N] = ANY OTHER OCCURRENCE OF ConditionalValue((^IITR76 = 'N'),^IITR77,NULL)

                    Data Elements:

                    RP.{ETPSeqNum}:^IITR74 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldEmploymentTerminationPayment.Amount

                    RP.{ETPSeqNum}:^IITR76 = tns:IncomeTax.PayAsYouGoWithholding.EmploymentTerminationPaymentType.Code

                    RP.{ETPSeqNum}:^IITR546 = tns:Identifiers.WithholdingPayerNumber.Identifier

                    RP.{ETPSeqNum}:^IITR75 = tns:Income.EmploymentTerminationPaymentTaxable.Amount

                    RP.{ETPSeqNum}:^IITR77 = tns:Identifiers.AustralianBusinessNumber.Identifier
                    */
                    List<IITR2018.RPETPSeqNumContext> etpContextsIITR77_630009 = report.RPETPSeqNumContextCollection.FindAll(f => f.IITR76 == "N" && f.IITR77 == rpETPSeqNumContext.IITR77);
                    List<IITR2018.RPETPSeqNumContext> etpContextsIITR546_630009 = report.RPETPSeqNumContextCollection.FindAll(f => f.IITR76 == "N" && f.IITR546 == rpETPSeqNumContext.IITR546);
                    
                    assertion = (rpETPSeqNumContext.IITR76 == "N" && ((rpETPSeqNumContext.IITR546 != null && (etpContextsIITR546_630009.Count > 0 ? etpContextsIITR546_630009.Sum(f => f.IITR75.GetValueOrDefault()) : 0) > 200000 && Convert.ToDouble(etpContextsIITR546_630009.Count > 0 ? etpContextsIITR546_630009.Sum(f => f.IITR74.GetValueOrDefault()) : 0) < (etpContextsIITR546_630009.Count > 0 ? Convert.ToDouble(etpContextsIITR546_630009.Sum(f => f.IITR75.GetValueOrDefault())) * 0.32 : 0)) || (rpETPSeqNumContext.IITR77 != null && (etpContextsIITR77_630009.Count > 0 ? etpContextsIITR77_630009.Sum(f => f.IITR75.GetValueOrDefault()) : 0) > 200000 && (etpContextsIITR77_630009.Count > 0 ? Convert.ToDouble(etpContextsIITR77_630009.Sum(f => f.IITR74.GetValueOrDefault())) : 0) < (etpContextsIITR77_630009.Count > 0 ? Convert.ToDouble(etpContextsIITR77_630009.Sum(f => f.IITR75.GetValueOrDefault())) * 0.32 : 0))));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITR.630009",
                            Severity = ProcessMessageSeverity.Warning,
                            Description = @"Check the Tax Withheld amount and the ETP code provided on the PAYG ETP Summary",
                            Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldEmploymentTerminationPayment.Amount[@contextRef='" + rpETPSeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.630009" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR74", Value = rpETPSeqNumContext.IITR74.GetValueOrDefault().ToString() });
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR75", Value = rpETPSeqNumContext.IITR75.GetValueOrDefault().ToString() });
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR76", Value = rpETPSeqNumContext.IITR76 });
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR77", Value = rpETPSeqNumContext.IITR77 });
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR546", Value = rpETPSeqNumContext.IITR546 });

                        response.Add(processMessage);
                    }

                    #endregion // VR.ATO.IITR.630009
                }
            }

            #endregion // Repeating RPETPSeqNumContext

            #region Repeating RPLOSSSeqNumContext

            if (report.RPLOSSSeqNumContextCollection != null)
            {
                foreach (IITR2018.RPLOSSSeqNumContext rplossSeqNumContext in report.RPLOSSSeqNumContextCollection)
                {
                    #region VR.ATO.IITR.400063

                    /*  VR.ATO.IITR.400063
                    The Sequence Container within the Reporting Party Loss context must be assigned sequentially beginning at 1 and must not exceed 3

                    Legacy Rule Format:
                    IF ([iitr573] <> SET(1-3)) OR ([iitr573] > COUNT(RP.{LOSSSeqNum}))
                       RETURN VALIDATION MESSAGE
                    ENDIF

                    Technical Business Rule Format:
                    (NotInSet(^IITR573, '1-3')) OR (^IITR573 > Count(^Context))

                    Data Elements:

                    ^Context = RP:RP.{LOSSSeqNum}

                    RP.{LOSSSeqNum}:^IITR573 = tns:Report.ItemOrder.Number
                    */
                    assertion = (!(IsMatch(rplossSeqNumContext.IITR573, @"^([1-3])*$", RegexOptions.IgnoreCase)) || Convert.ToInt32(rplossSeqNumContext.IITR573) > Count(report.RPLOSSSeqNumContextCollection));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITR.400063",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"The Sequence Container within the Reporting Party Loss context is incorrect",
                            LongDescription = @"The Sequence Container within the Reporting Party Loss context must be assigned sequentially beginning at 1 and must not exceed 3",
                            Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLOSSSeqNumContextCollection == null ? 0 : (report.RPLOSSSeqNumContextCollection.Count() == 0 ? 0 : report.RPLOSSSeqNumContextCollection.Last().LastOccurrenceIndex)),
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.400063" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR573", Value = rplossSeqNumContext.IITR573 });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IITR.400063
                }
            }

            #endregion // Repeating RPLOSSSeqNumContext

            #region Repeating RPPHISeqNumContext

            if (report.RPPHISeqNumContextCollection != null)
            {
                foreach (IITR2018.RPPHISeqNumContext rpPHISeqNumContext in report.RPPHISeqNumContextCollection)
                {
                    #region VR.ATO.IITR.000604

                    /*  VR.ATO.IITR.000604
                    Private health insurance policy has an entry with Tax claim code "Claiming your spouses share of rebate" then there must be a matching entry with Tax claim code "Claiming your share of rebate". The number of lines with tax claim code "Claiming your spouses share of rebate" must be equal to the number of lines with tax claim code "Claiming your share of rebate" for a particular policy, but this cannot be enforced due to technical constraints.

                    Legacy Rule Format:
                    IF ([iitr185:Claiming your spouses share of rebate] <> NULL) AND ((RP.{PHISeqNum} WHERE ([iitr185] = "Claiming your share of rebate" AND [iitr180: Claiming your spouses share of rebate] = [iitr180] AND [iitr181: Claiming your spouses share of rebate] = [iitr181])) = NULL)
                       RETURN VALIDATION MESSAGE
                    ENDIF

                    [iitr185:Claiming your spouses share of rebate] = ([iitr185] = "Claiming your spouses share of rebate")
                    [iitr180:Claiming your spouses share of rebate] = ([iitr180] WHERE [iitr185] = "Claiming your spouses share of rebate")
                    [iitr181:Claiming your spouses share of rebate] = ([iitr181] WHERE [iitr185] = "Claiming your spouses share of rebate")

                    Technical Business Rule Format:
                    ([iitr185:Claiming your spouses share of rebate] <> NULL) AND ((RP.{PHISeqNum} WHERE (^IITR185 = 'Claiming your share of rebate' AND [iitr180: Claiming your spouses share of rebate] = ^IITR180 AND [iitr181: Claiming your spouses share of rebate] = ^IITR181)) = NULL)           [iitr185:Claiming your spouses share of rebate] = (^IITR185 = 'Claiming your spouses share of rebate') [iitr180:Claiming your spouses share of rebate] = (^IITR180 WHERE ^IITR185 = 'Claiming your spouses share of rebate') [iitr181:Claiming your spouses share of rebate] = (^IITR181 WHERE ^IITR185 = 'Claiming your spouses share of rebate')

                    Data Elements:

                    RP.{PHISeqNum}:^IITR185 = tns:IncomeTax.HealthInsurance.TaxClaim.Code

                    RP.{PHISeqNum}:^IITR180 = tns:Identifiers.HealthInsuranceProvider.Identifier

                    RP.{PHISeqNum}:^IITR181 = tns:Identifiers.HealthInsuranceMembership.Identifier
                    */
                    assertion = (rpPHISeqNumContext.IITR185 == "Claiming your spouses share of rebate" && report.RPPHISeqNumContextCollection.Find(f => f.IITR185 == "Claiming your share of rebate" && f.IITR180 == rpPHISeqNumContext.IITR180 && f.IITR181 == rpPHISeqNumContext.IITR181) == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITR.000604",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Private health insurance details must match where claiming both your and spouse's share of rebate.",
                            Location = "/xbrli:xbrl/tns:IncomeTax.HealthInsurance.TaxClaim.Code[@contextRef='" + rpPHISeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.000604" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR180", Value = rpPHISeqNumContext.IITR180 });
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR181", Value = rpPHISeqNumContext.IITR181 });
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR185", Value = rpPHISeqNumContext.IITR185 });

                        response.Add(processMessage);
                    }

                    #endregion // VR.ATO.IITR.000604

                    #region VR.ATO.IITR.210034

                    /*  VR.ATO.IITR.210034
                    If two or more rows have been entered for one policy, tax claim codes must be the same on all rows except where the tax claim code is "Claiming your spouses share of rebate" or "Covered as dependent child" in any of those rows. Tax claim code(s) must be corrected.
    
                    Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{PHISeqNum})
                    IF ([iitr180] =  ANY OTHER OCCURRENCE OF ([iitr180] WHERE  [iitr185] <> SET("Claiming your spouses share of rebate","Covered as dependent child"))) AND ([iitr181] = [iitr181:Other]) AND ([iitr185] <> SET("Claiming your spouses share of rebate","Covered as dependent child")) AND ([iitr185] <> [iitr185:Other])
                       RETURN VALIDATION MESSAGE
                    ENDIF
            
            
                    [iitr180:Other] = ANY OTHER OCCURRENCE OF ([iitr180] WHERE  [iitr185] <> SET("Claiming your spouses share of rebate","Covered as dependent child"))
                    [iitr181:Other] = ANY OTHER OCCURRENCE OF ([iitr181] WHERE  [iitr185] <> SET("Claiming your spouses share of rebate","Covered as dependent child"))
                    [iitr185:Other] = ANY OTHER OCCURRENCE OF ([iitr185] WHERE  [iitr185] <> SET("Claiming your spouses share of rebate","Covered as dependent child"))

                    Technical Business Rule Format:
                    (HasDuplicateValues(^IITR180) WHERE  NotInSet(^IITR185, '"Claiming your spouses share of rebate","Covered as dependent child"'))) AND (^IITR181 = [iitr181:Other]) AND (NotInSet(^IITR185, '"Claiming your spouses share of rebate","Covered as dependent child"')) AND (^IITR185 <> [iitr185:Other])            [iitr180:Other] = ANY OTHER OCCURRENCE OF (^IITR180 WHERE  NotInSet(^IITR185, '"Claiming your spouses share of rebate","Covered as dependent child"')) [iitr181:Other] = ANY OTHER OCCURRENCE OF (^IITR181 WHERE  NotInSet(^IITR185, '"Claiming your spouses share of rebate","Covered as dependent child"')) [iitr185:Other] = ANY OTHER OCCURRENCE OF (^IITR185 WHERE  NotInSet(^IITR185, '"Claiming your spouses share of rebate","Covered as dependent child"'))
    
                    Data Elements:
    
                    RP.{PHISeqNum}:^IITR185 = tns:IncomeTax.HealthInsurance.TaxClaim.Code
    
                    RP.{PHISeqNum}:^IITR180 = tns:Identifiers.HealthInsuranceProvider.Identifier
    
                    RP.{PHISeqNum}:^IITR181 = tns:Identifiers.HealthInsuranceMembership.Identifier
                    */
                    assertion = (report.RPPHISeqNumContextCollection.Find(f => f.IITR180 == rpPHISeqNumContext.IITR180 && f.IITR181 == rpPHISeqNumContext.IITR181 && f.IITR185 != rpPHISeqNumContext.IITR185 && f.IITR185 != "Claiming your spouses share of rebate" && f.IITR185 != "Covered as dependent child") != null && rpPHISeqNumContext.IITR185 != "Claiming your spouses share of rebate" && rpPHISeqNumContext.IITR185 != "Covered as dependent child");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITR.210034",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Tax claim code(s) against the same Private Health Insurance policy are incorrect",
                            LongDescription = @"When multiple Private Health Insurance details have been entered against the same policy ('Private health insurance - Health insurer ID' and 'Private health insurance - Health insurer ID'), the 'Tax claim code' must be the same across all the policy entries, except where the 'Tax claim code' is either ""Claiming your spouses share of rebate"" or ""Covered as dependent child""",
                            Location = "/xbrli:xbrl/tns:IncomeTax.HealthInsurance.TaxClaim.Code[@contextRef='" + rpPHISeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.210034" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR180", Value = rpPHISeqNumContext.IITR180 });
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR181", Value = rpPHISeqNumContext.IITR181 });
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR185", Value = rpPHISeqNumContext.IITR185 });

                        response.Add(processMessage);
                    }

                    #endregion // VR.ATO.IITR.210034
                }
            }

            #endregion // Repeating RPPHISeqNumContext
            
            #region Repeating RPSLSPSeqNumContext

            if (report.RPSLSPSeqNumContextCollection != null)
            {
                foreach (IITR2018.RPSLSPSeqNumContext rpslspSeqNumContext in report.RPSLSPSeqNumContextCollection)
                {
                    #region VR.ATO.IITR.100052

                    /*  VR.ATO.IITR.100052
                    Life benefit superannuation lump sums containing taxed elements paid on or after the taxpayer's 60th birthday are non-assessable and non-exempt income, and should not be declared in the tax return.
    
                    Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SLSPSeqNum})
                    IF ([iitr97] >= ConvertToDate(Day([iitr29]), MonthAsInt([iitr29]), Year([iitr29]) + 60)) AND ([iitr99] > 0) AND ([iitr102] = NULL OR [iitr102] = FALSE)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^IITR97 >= ConvertToDate(Day(^IITR29), MonthAsInt(^IITR29), Year(^IITR29) + 60)) AND (^IITR99 > 0) AND (^IITR102 = NULL OR ^IITR102 = FALSE)
            
                    Data Elements:
            
                    RP.{SLSPSeqNum}:^IITR97 = tns:Income.Superannuation.LumpSumPayment.Date
            
                    RP:^IITR29 = tns:PersonDemographicDetails.Birth.Date
            
                    RP.{SLSPSeqNum}:^IITR99 = tns:Income.Superannuation.LumpSumPaymentTaxableComponentTaxedElement.Amount
            
                    RP.{SLSPSeqNum}:^IITR102 = tns:Income.Superannuation.DeathBenefitPaymentNonDependant.Indicator
                    */
                    assertion = ((report.IITR29 != null ? rpslspSeqNumContext.IITR97.GetValueOrDefault() >= report.IITR29.Value.AddYears(60) : false) && rpslspSeqNumContext.IITR99.GetValueOrDefault() > 0 && (rpslspSeqNumContext.IITR102 == null || rpslspSeqNumContext.IITR102 == false));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITR.100052",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Taxed elements paid on or after the taxpayers 60th birthday must not be declared",
                            LongDescription = @"Life benefit superannuation lump sums containing taxed elements paid on or after the taxpayer's 60th birthday are non-assessable and non-exempt income, and must not be declared in the tax return",
                            Location = "/xbrli:xbrl/tns:Income.Superannuation.LumpSumPayment.Date[@contextRef='" + rpslspSeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.100052" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR97", Value = GetValueOrEmpty(rpslspSeqNumContext.IITR97) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR29", Value = GetValueOrEmpty(report.IITR29) });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR99", Value = rpslspSeqNumContext.IITR99.GetValueOrDefault().ToString() });

                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR102", Value = GetValueOrEmpty(rpslspSeqNumContext.IITR102) });

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IITR.100052

                    #region VR.ATO.IITR.630011

                    /*  VR.ATO.IITR.630011
                    Where the age of tax payer is less than 55 on 30 June of the reporting period; and Total tax withheld from SLS payments, from the same Payer ABN, is more than the sum of First $1,445,000 of the total SLS untaxed amounts, from the same payer ABN, * 32% (30% SLS rate 3 + 2% Medicare), Plus First $1,445,000 of the total SLS untaxed amounts, from the same payer ABN, above $1,445,000 * 47% (45% SLS rate 4 + 2% Medicare), check Australian superannuation lump sum payments "Tax withheld amount"

                    Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SLSPSeqNum})
                    IF ([iitr29] > CONVERT_TO_DATE(30,6,[iitr10] - 55)) AND ([iitr101] <> NULL) AND ([iitr99] = SET(NULL,0)) AND (((SUM(ConditionalValue([iitr101] = [iitr101:0],[iitr100],0)) <= 1445000) AND (SUM(ConditionalValue([iitr101] = [iitr101:0],[iitr98],0)) > (SUM(ConditionalValue([iitr101] = [iitr101:0],[iitr100],0)) * 0.32))) OR ((SUM(ConditionalValue([iitr101] = [iitr101:0],[iitr100],0)) > 1445000) AND (SUM(ConditionalValue([iitr101] = [iitr101:0],[iitr98],0)) > (1445000 * 0.32 + ((SUM(ConditionalValue([iitr101] = [iitr101:0],[iitr100],0)) - 1445000) * 0.47)))))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    [iitr101:0] = ANY OTHER OCCURRENCE OF ConditionalValue([iitr99] = SET(NULL,0),[iitr101],NULL)

                    Technical Business Rule Format:
                    (^IITR29 > ConvertToDate(30, 6, ^IITR10 - 55)) AND (^IITR101 <> NULL) AND (InSet(^IITR99, 'NULL,0')) AND (((Sum(ConditionalValue(^IITR101 = [iitr101:0],^IITR100,0)) <= 1445000) AND (Sum(ConditionalValue(^IITR101 = [iitr101:0],^IITR98,0)) > (Sum(ConditionalValue(^IITR101 = [iitr101:0],^IITR100,0)) * 0.32))) OR ((Sum(ConditionalValue(^IITR101 = [iitr101:0],^IITR100,0)) > 1445000) AND (Sum(ConditionalValue(^IITR101 = [iitr101:0],^IITR98,0)) > (1445000 * 0.32 + ((Sum(ConditionalValue(^IITR101 = [iitr101:0],^IITR100,0)) - 1445000) * 0.47)))))          [iitr101:0] = ANY OTHER OCCURRENCE OF ConditionalValue(InSet(^IITR99, 'NULL,0'),^IITR101,NULL)

                    Data Elements:

                    RP.{SLSPSeqNum}:^IITR98 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldSuperannuationLumpSumPayment.Amount

                    RP:^IITR29 = tns:PersonDemographicDetails.Birth.Date

                    RP:^IITR10 = tns:Report.TargetFinancial.Year

                    RP.{SLSPSeqNum}:^IITR101 = tns:Identifiers.AustralianBusinessNumber.Identifier

                    RP.{SLSPSeqNum}:^IITR99 = tns:Income.Superannuation.LumpSumPaymentTaxableComponentTaxedElement.Amount

                    RP.{SLSPSeqNum}:^IITR100 = tns:Income.Superannuation.LumpSumPaymentTaxableComponentUntaxedElement.Amount
                    */
                    List<IITR2018.RPSLSPSeqNumContext> slspContextsIITR101_630011 = report.RPSLSPSeqNumContextCollection.FindAll(f => (f.IITR99 == null || f.IITR99 == 0) && f.IITR101 == rpslspSeqNumContext.IITR101);

                    assertion = (report.IITR29 != null && (report.IITR10 != null ? report.IITR29 > ConvertToDate(30, 6, report.IITR10.Value - 55) : false) && rpslspSeqNumContext.IITR101 != null && (rpslspSeqNumContext.IITR99 == null || rpslspSeqNumContext.IITR99 == 0) && (((slspContextsIITR101_630011.Count > 0 ? slspContextsIITR101_630011.Sum(f => f.IITR100.GetValueOrDefault()) : 0) <= 1445000 && Convert.ToDouble(slspContextsIITR101_630011.Count > 0 ? slspContextsIITR101_630011.Sum(f => f.IITR98.GetValueOrDefault()) : 0) > (Convert.ToDouble(slspContextsIITR101_630011.Count > 0 ? slspContextsIITR101_630011.Sum(f => f.IITR100.GetValueOrDefault()) : 0) * 0.32)) || ((slspContextsIITR101_630011.Count > 0 ? slspContextsIITR101_630011.Sum(f => f.IITR100.GetValueOrDefault()) : 0) > 1445000 && Convert.ToDouble(slspContextsIITR101_630011.Count > 0 ? slspContextsIITR101_630011.Sum(f => f.IITR98.GetValueOrDefault()) : 0) > (1445000 * 0.32 + ((Convert.ToDouble(slspContextsIITR101_630011.Count > 0 ? slspContextsIITR101_630011.Sum(f => f.IITR100.GetValueOrDefault()) : 0) - 1445000) * 0.47)))));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITR.630011",
                            Severity = ProcessMessageSeverity.Warning,
                            Description = @"Check the Tax Withheld amount matches Total tax Withheld from the PAYG SLS summary.",
                            LongDescription = @"Check the Tax Withheld amount matches Total tax Withheld from the PAYG SLS summary; if payment is from a taxed or untaxed element and taxpayer's age when payment was received.",
                            Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldSuperannuationLumpSumPayment.Amount[@contextRef='" + rpslspSeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.630011" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR10", Value = report.IITR10.GetValueOrDefault().ToString() });
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR29", Value = report.IITR29.GetValueOrDefault().ToString() });
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR98", Value = rpslspSeqNumContext.IITR98.GetValueOrDefault().ToString() });
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR99", Value = rpslspSeqNumContext.IITR99.GetValueOrDefault().ToString() });
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR100", Value = rpslspSeqNumContext.IITR100.GetValueOrDefault().ToString() });
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR101", Value = rpslspSeqNumContext.IITR101 });

                        response.Add(processMessage);
                    }

                    #endregion // VR.ATO.IITR.630011

                    #region VR.ATO.IITR.630012

                    /*  VR.ATO.IITR.630012
                    Where the age of the tax payer is greater than 60 on 30 June of the reporting period; and Total tax withheld from SLS payments, from the same Payer ABN, is more than the sum of First $1,445,000 of the total SLS untaxed amounts, from the same payer ABN, * 17% (15% sls rate 1 + 2% Medicare), Plus First $1,445,000 of the total SLS untaxed amounts, from the same payer ABN, above $1,445,000 * 47% (45% sls rate 4 + 2% Medicare), check Australian superannuation lump sum payments "Tax withheld amount"

                    Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SLSPSeqNum})
                    IF ([iitr29] <= CONVERT_TO_DATE(30,6,[iitr10] - 60)) AND ([iitr101] <> NULL) AND ([iitr99] = SET(NULL,0)) AND (((SUM(ConditionalValue([iitr101] = [iitr101:0],[iitr100],0)) <= 1445000) AND (SUM(ConditionalValue([iitr101] = [iitr101:0],[iitr98],0)) > (SUM(ConditionalValue([iitr101] = [iitr101:0],[iitr100],0)) * 0.17))) OR ((SUM(ConditionalValue([iitr101] = [iitr101:0],[iitr100],0)) > 1445000) AND (SUM(ConditionalValue([iitr101] = [iitr101:0],[iitr98],0)) > (1445000 * 0.17 + ((SUM(ConditionalValue([iitr101] = [iitr101:0],[iitr100],0)) - 1445000) * 0.47)))))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    [iitr101:0] = ANY OTHER OCCURRENCE OF ConditionalValue([iitr99] = SET(NULL,0),[iitr101],NULL)

                    Technical Business Rule Format:
                    (^IITR29 <= ConvertToDate(30, 6, ^IITR10 - 60)) AND (^IITR101 <> NULL) AND (InSet(^IITR99, 'NULL,0')) AND (((Sum(ConditionalValue(^IITR101 = [iitr101:0],^IITR100,0)) <= 1445000) AND (Sum(ConditionalValue(^IITR101 = [iitr101:0],^IITR98,0)) > (Sum(ConditionalValue(^IITR101 = [iitr101:0],^IITR100,0)) * 0.17))) OR ((Sum(ConditionalValue(^IITR101 = [iitr101:0],^IITR100,0)) > 1445000) AND (Sum(ConditionalValue(^IITR101 = [iitr101:0],^IITR98,0)) > (1445000 * 0.17 + ((Sum(ConditionalValue(^IITR101 = [iitr101:0],^IITR100,0)) - 1445000) * 0.47)))))          [iitr101:0] = ANY OTHER OCCURRENCE OF ConditionalValue(InSet(^IITR99, 'NULL,0'),^IITR101,NULL)

                    Data Elements:

                    RP.{SLSPSeqNum}:^IITR98 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldSuperannuationLumpSumPayment.Amount

                    RP:^IITR29 = tns:PersonDemographicDetails.Birth.Date

                    RP:^IITR10 = tns:Report.TargetFinancial.Year

                    RP.{SLSPSeqNum}:^IITR101 = tns:Identifiers.AustralianBusinessNumber.Identifier

                    RP.{SLSPSeqNum}:^IITR99 = tns:Income.Superannuation.LumpSumPaymentTaxableComponentTaxedElement.Amount

                    RP.{SLSPSeqNum}:^IITR100 = tns:Income.Superannuation.LumpSumPaymentTaxableComponentUntaxedElement.Amount
                    */
                    List<IITR2018.RPSLSPSeqNumContext> slspContextsIITR101_630012 = report.RPSLSPSeqNumContextCollection.FindAll(f => (f.IITR99 == null || f.IITR99 == 0) && f.IITR101 == rpslspSeqNumContext.IITR101);

                    assertion = (report.IITR29 != null && (report.IITR10 != null ? report.IITR29 <= ConvertToDate(30, 6, report.IITR10.Value - 60) : false) && rpslspSeqNumContext.IITR101 != null && (rpslspSeqNumContext.IITR99 == null || rpslspSeqNumContext.IITR99 == 0) && (((slspContextsIITR101_630011.Count > 0 ? slspContextsIITR101_630011.Sum(f => f.IITR100.GetValueOrDefault()) : 0) <= 1445000 && Convert.ToDouble(slspContextsIITR101_630011.Count > 0 ? slspContextsIITR101_630011.Sum(f => f.IITR98.GetValueOrDefault()) : 0) > (Convert.ToDouble(slspContextsIITR101_630011.Count > 0 ? slspContextsIITR101_630011.Sum(f => f.IITR100.GetValueOrDefault()) : 0) * 0.17)) || ((slspContextsIITR101_630011.Count > 0 ? slspContextsIITR101_630011.Sum(f => f.IITR100.GetValueOrDefault()) : 0) > 1445000 && Convert.ToDouble(slspContextsIITR101_630011.Count > 0 ? slspContextsIITR101_630011.Sum(f => f.IITR98.GetValueOrDefault()) : 0) > (1445000 * 0.17) + ((Convert.ToDouble(slspContextsIITR101_630011.Count > 0 ? slspContextsIITR101_630011.Sum(f => f.IITR100.GetValueOrDefault()) : 0) - 1445000) * 0.47))));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITR.630012",
                            Severity = ProcessMessageSeverity.Warning,
                            Description = @"Check the Tax Withheld amount matches Total tax Withheld from the PAYG SLS summary.",
                            LongDescription = @"Check the Tax Withheld amount matches Total tax Withheld from the PAYG SLS summary; if payment is from a taxed or untaxed element and taxpayer's age when payment was received.",
                            Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldSuperannuationLumpSumPayment.Amount[@contextRef='" + rpslspSeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.630012" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR10", Value = report.IITR10.GetValueOrDefault().ToString() });
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR29", Value = report.IITR29.GetValueOrDefault().ToString() });
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR98", Value = rpslspSeqNumContext.IITR98.GetValueOrDefault().ToString() });
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR99", Value = rpslspSeqNumContext.IITR99.GetValueOrDefault().ToString() });
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR100", Value = rpslspSeqNumContext.IITR100.GetValueOrDefault().ToString() });
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR101", Value = rpslspSeqNumContext.IITR101 });

                        response.Add(processMessage);
                    }

                    #endregion // VR.ATO.IITR.630012

                    #region VR.ATO.IITR.630013

                    /*  VR.ATO.IITR.630013
                    Where the age of the tax payer is greater than 55 and less than 60 on 30 June of the reporting period; and Total tax withheld from SLS payments, from the same Payer ABN, is more than the sum of First $200,000 of the total SLS untaxed amounts, from the same payer ABN, * 17% (15% sls rate 1 + 2% Medicare), Plus amount between $200,000 and up to  $1,445,000 of the total SLS untaxed amounts, from the same payer ABN, * 32% (30% sls rate 3 + 2% Medicare), Plus First $1,445,000 of the total SLS untaxed amounts, from the same payer ABN, above $1,445,000 * 47% (45% sls rate 4 + 2% Medicare), check Australian superannuation lump sum payments "Tax withheld amount"

                    Legacy Rule Format:
                    WHERE IN CONTEXT(RP.{SLSPSeqNum})
                    IF ([iitr29] > CONVERT_TO_DATE(30,6,[iitr10] - 60)) AND ([iitr29] <= CONVERT_TO_DATE(30,6,[iitr10] - 55)) AND ([iitr101] <> NULL) AND ([iitr99] = SET(NULL,0)) AND (((SUM(ConditionalValue([iitr101] = [iitr101:0],[iitr100],0)) <= 200000) AND (SUM(ConditionalValue([iitr101] = [iitr101:0],[iitr98],0)) > (SUM(ConditionalValue([iitr101] = [iitr101:0],[iitr100],0)) * 0.17))) OR ((SUM(ConditionalValue([iitr101] = [iitr101:0],[iitr100],0)) > 200000) AND (SUM(ConditionalValue([iitr101] = [iitr101:0],[iitr100],0)) <= 1445000) AND (SUM(ConditionalValue([iitr101] = [iitr101:0],[iitr98],0)) > (200000 * 0.17 + ((SUM(ConditionalValue([iitr101] = [iitr101:0],[iitr100],0)) - 200000) * 0.32)))) OR ((SUM(ConditionalValue([iitr101] = [iitr101:0],[iitr100],0)) > 1445000) AND (SUM(ConditionalValue([iitr101] = [iitr101:0],[iitr98],0)) > (200000 * 0.17 + (1445000 - 200000) * 0.32 + ((SUM(ConditionalValue([iitr101] = [iitr101:0],[iitr100],0)) - 1445000) * 0.47)))))
                       RETURN VALIDATION MESSAGE
                    ENDIF

                    [iitr101:0] = ANY OTHER OCCURRENCE OF ConditionalValue([iitr99] = SET(NULL,0),[iitr101],NULL)

                    Technical Business Rule Format:
                    (^IITR29 > ConvertToDate(30, 6, ^IITR10 - 60)) AND (^IITR29 <= ConvertToDate(30, 6, ^IITR10 - 55)) AND (^IITR101 <> NULL) AND (InSet(^IITR99, 'NULL,0')) AND (((Sum(ConditionalValue(^IITR101 = [iitr101:0],^IITR100,0)) <= 200000) AND (Sum(ConditionalValue(^IITR101 = [iitr101:0],^IITR98,0)) > (Sum(ConditionalValue(^IITR101 = [iitr101:0],^IITR100,0)) * 0.17))) OR ((Sum(ConditionalValue(^IITR101 = [iitr101:0],^IITR100,0)) > 200000) AND (Sum(ConditionalValue(^IITR101 = [iitr101:0],^IITR100,0)) <= 1445000) AND (Sum(ConditionalValue(^IITR101 = [iitr101:0],^IITR98,0)) > (200000 * 0.17 + ((Sum(ConditionalValue(^IITR101 = [iitr101:0],^IITR100,0)) - 200000) * 0.32)))) OR ((Sum(ConditionalValue(^IITR101 = [iitr101:0],^IITR100,0)) > 1445000) AND (Sum(ConditionalValue(^IITR101 = [iitr101:0],^IITR98,0)) > (200000 * 0.17 + (1445000 - 200000) * 0.32 + ((Sum(ConditionalValue(^IITR101 = [iitr101:0],^IITR100,0)) - 1445000) * 0.47)))))           [iitr101:0] = ANY OTHER OCCURRENCE OF ConditionalValue(InSet(^IITR99, 'NULL,0'),^IITR101,NULL)

                    Data Elements:

                    RP.{SLSPSeqNum}:^IITR98 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldSuperannuationLumpSumPayment.Amount

                    RP:^IITR29 = tns:PersonDemographicDetails.Birth.Date

                    RP:^IITR10 = tns:Report.TargetFinancial.Year

                    RP.{SLSPSeqNum}:^IITR101 = tns:Identifiers.AustralianBusinessNumber.Identifier

                    RP.{SLSPSeqNum}:^IITR99 = tns:Income.Superannuation.LumpSumPaymentTaxableComponentTaxedElement.Amount

                    RP.{SLSPSeqNum}:^IITR100 = tns:Income.Superannuation.LumpSumPaymentTaxableComponentUntaxedElement.Amount
                    */
                    List<IITR2018.RPSLSPSeqNumContext> slspContextsIITR101_630013 = report.RPSLSPSeqNumContextCollection.FindAll(f => (f.IITR99 == null || f.IITR99 == 0) && f.IITR101 == rpslspSeqNumContext.IITR101);

                    assertion = (report.IITR10 != null && report.IITR29 > ConvertToDate(30, 6, report.IITR10.Value - 60) && (report.IITR10 != null ? report.IITR29 <= ConvertToDate(30, 6, report.IITR10.Value - 55) : false) && rpslspSeqNumContext.IITR101 != null && (rpslspSeqNumContext.IITR99 == null || rpslspSeqNumContext.IITR99 == 0) && (((HasDuplicateValues(slspContextsIITR101_630011) ? slspContextsIITR101_630011.Sum(f => f.IITR100.GetValueOrDefault()) : 0) <= 200000 && Convert.ToDouble(HasDuplicateValues(slspContextsIITR101_630011) ? slspContextsIITR101_630011.Sum(f => f.IITR98.GetValueOrDefault()) : 0) > (Convert.ToDouble(HasDuplicateValues(slspContextsIITR101_630011) ? slspContextsIITR101_630011.Sum(f => f.IITR100.GetValueOrDefault()) : 0) * 0.17)) || ((HasDuplicateValues(slspContextsIITR101_630011) ? slspContextsIITR101_630011.Sum(f => f.IITR100.GetValueOrDefault()) : 0) > 200000 && HasDuplicateValues(slspContextsIITR101_630011) ? slspContextsIITR101_630011.Sum(f => f.IITR100.GetValueOrDefault()) : 0) <= 1445000 && Convert.ToDouble(HasDuplicateValues(slspContextsIITR101_630011) ? slspContextsIITR101_630011.Sum(f => f.IITR98.GetValueOrDefault()) : 0) > (200000 * 0.17) + ((Convert.ToDouble(HasDuplicateValues(slspContextsIITR101_630011) ? slspContextsIITR101_630011.Sum(f => f.IITR100.GetValueOrDefault()) : 0) - 200000) * 0.32)) || ((HasDuplicateValues(slspContextsIITR101_630011) ? slspContextsIITR101_630011.Sum(f => f.IITR100.GetValueOrDefault()) : 0) > 1445000 && Convert.ToDouble(HasDuplicateValues(slspContextsIITR101_630011) ? slspContextsIITR101_630011.Sum(f => f.IITR98.GetValueOrDefault()) : 0) > (200000 * 0.17 + (144500 - 200000 * 0.32)) + ((Convert.ToDouble(HasDuplicateValues(slspContextsIITR101_630011) ? slspContextsIITR101_630011.Sum(f => f.IITR100.GetValueOrDefault()) : 0) - 144500) * 0.47)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITR.630013",
                            Severity = ProcessMessageSeverity.Warning,
                            Description = @"Check the Tax Withheld amount matches Total tax Withheld from the PAYG SLS summary.",
                            LongDescription = @"Check the Tax Withheld amount matches Total tax Withheld from the PAYG SLS summary; if payment is from a taxed or untaxed element and taxpayer's age when payment was received.",
                            Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldSuperannuationLumpSumPayment.Amount[@contextRef='" + rpslspSeqNumContext.Id + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.630013" } },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR10", Value = report.IITR10.GetValueOrDefault().ToString() });
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR29", Value = report.IITR29.GetValueOrDefault().ToString() });
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR98", Value = rpslspSeqNumContext.IITR98.GetValueOrDefault().ToString() });
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR99", Value = rpslspSeqNumContext.IITR99.GetValueOrDefault().ToString() });
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR100", Value = rpslspSeqNumContext.IITR100.GetValueOrDefault().ToString() });
                        processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IITR101", Value = rpslspSeqNumContext.IITR101 });

                        response.Add(processMessage);
                    }

                    #endregion // VR.ATO.IITR.630012
                }
            }

            #endregion // Repeating RPSLSPSeqNumContext

            return response;
        }
        
        #region VR.ATO.IITR.100031

        protected bool VRATOIITR100031(IITR2018 report, IITR2018.RPLOSSSeqNumContext rplossSeqNumContext)
        {
            /*  VR.ATO.IITR.100031
            For a resident taxpayer, where the sum of taxable income or loss and the net loss amounts that exceeds or equal to zero, the reported income for the purposes of the non-commercial losses income test is equal to $250,000 or more. Ensure loss codes used and/or amounts reported at the relevant labels are correct. A copy of the formula used to calculate income for this test can be found at www.ato.gov.au/ncl
    
            Legacy Rule Format:
            IF [iitr19] = TRUE AND ANY OCCURRENCE OF([iitr482]) = SET(1,2,3,4) AND ([iitr157] + SUM(ConditionalValue([iitr482] <> SET(0,5,8), [iitr487], 0))) >= 0 AND ([iitr157] + SUM(ConditionalValue([iitr482] <> SET(0,5,8), [iitr487], 0)) + ConditionalValue(([iitr490] + [iitr491]) > 3772, ([iitr490] + [iitr491]), 0) + [iitr204] + [iitr205] + [iitr201] + SUM([iitr362]) - ([iitr153] - [iitr154]) - ([iitr155] - [iitr156])) >= 250000
                RETURN VALIDATION MESSAGE
            ENDIF
        
            Technical Business Rule Format:
            ^IITR19 = TRUE AND AnyOccurrence(^IITR482, InSet(^IITR482, '1,2,3,4')) AND (^IITR157 + Sum(ConditionalValue(NotInSet(^IITR482, '0,5,8'), ^IITR487, 0))) >= 0 AND (^IITR157 + Sum(ConditionalValue(NotInSet(^IITR482, '0,5,8'), ^IITR487, 0)) + ConditionalValue((^IITR490 + ^IITR491) > 3772, (^IITR490 + ^IITR491), 0) + ^IITR204 + ^IITR205 + ^IITR201 + Sum(^IITR362) - (^IITR153 - ^IITR154) - (^IITR155 - ^IITR156)) >= 250000
            
            Data Elements:
            
            RP.{LOSSSeqNum}:^IITR487 = tns:Tax.Losses.Net.Amount
            
            RP:^IITR19 = tns:Residency.TaxPurposesPersonStatus.Indicator
            
            RP.{LOSSSeqNum}:^IITR482 = tns:IncomeTax.LossesType.Code
            
            RP:^IITR157 = tns:Income.Taxable.Amount
            
            RP.Eligible:^IITR490 = tns:Income.FringeBenefitsReportable.Amount
            
            RP.Ineligible:^IITR491 = tns:Income.FringeBenefitsReportable.Amount
            
            RP:^IITR204 = tns:Tax.Losses.FinancialInvestmentNet.Amount
            
            RP:^IITR205 = tns:Tax.Losses.RentalPropertyNet.Amount
            
            RP:^IITR201 = tns:SuperannuationContribution.EmployerReportable.Amount
            
            RP.{SCSeqNum}:^IITR362 = tns:IncomeTax.Deduction.SuperannuationContribution.Amount
            
            RP:^IITR153 = tns:IncomeTax.LossesBroughtForwardPrimaryProduction.Amount
            
            RP:^IITR154 = tns:IncomeTax.LossesClaimedPrimaryProduction.Amount
            
            RP:^IITR155 = tns:IncomeTax.LossesBroughtForwardNonPrimaryProduction.Amount
            
            RP:^IITR156 = tns:IncomeTax.LossesClaimedNonPrimaryProduction.Amount
            */
            List<IITR2018.RPLOSSSeqNumContext> lossContextsIITR482_100031 = (report.RPLOSSSeqNumContextCollection != null ? report.RPLOSSSeqNumContextCollection.FindAll(f => !IsMatch(f.IITR482, "^(0|5|8)$")) : new List<IITR2018.RPLOSSSeqNumContext>());

            bool assertion = (report.IITR19 == true && IsMatch(rplossSeqNumContext.IITR482, "^([1-4])$") && (report.IITR157.GetValueOrDefault() + (lossContextsIITR482_100031.Count > 0 ? lossContextsIITR482_100031.Sum(f => f.IITR487.GetValueOrDefault()) : 0)) >= 0 && (report.IITR157.GetValueOrDefault() + (lossContextsIITR482_100031.Count > 0 ? lossContextsIITR482_100031.Sum(f => f.IITR487.GetValueOrDefault()) : 0) + (report.IITR490.GetValueOrDefault() + report.IITR491.GetValueOrDefault() > 3772 ? report.IITR490.GetValueOrDefault() + report.IITR491.GetValueOrDefault() : 0) + report.IITR204.GetValueOrDefault() + report.IITR205.GetValueOrDefault() + report.IITR201.GetValueOrDefault() + (report.RPSCSeqNumContextCollection != null ? report.RPSCSeqNumContextCollection.Sum(f => f.IITR362.GetValueOrDefault()) : 0) - (report.IITR153.GetValueOrDefault() - report.IITR154.GetValueOrDefault()) - (report.IITR155.GetValueOrDefault() - report.IITR156.GetValueOrDefault())) >= 250000);

            return assertion;
        }

        #endregion // VR.ATO.IITR.100031

        #region VR.ATO.IITR.100032

        protected bool VRATOIITR100032(IITR2018 report, IITR2018.RPLOSSSeqNumContext rplossSeqNumContext)
        {
            /*  VR.ATO.IITR.100032
            For a resident taxpayer where the sum of taxable income or loss and the net loss amounts is less than zero, the reported income for the purposes of the non-commercial losses income test is equal to $250,000 or more. Ensure loss codes used and/or amounts reported at the relevant labels are correct. A copy of the formula used to calculate income for this test can be found at www.ato.gov.au/ncl
    
            Legacy Rule Format:
            IF [iitr19] = TRUE AND ANY OCCURRENCE OF([iitr482]) = SET(1,2,3,4) AND ([iitr157] + SUM(ConditionalValue([iitr482] <> SET(0,8), [iitr487], 0))) < 0 AND (ConditionalValue(([iitr490] + [iitr491]) > 3772, ([iitr490] + [iitr491]), 0) + [iitr204] + [iitr205] + [iitr201] + SUM([iitr362]) - ([iitr153] - [iitr154]) - ([iitr155] - [iitr156])) >= 250000
                RETURN VALIDATION MESSAGE
            ENDIF
        
            Technical Business Rule Format:
            ^IITR19 = TRUE AND AnyOccurrence(^IITR482, InSet(^IITR482, '1,2,3,4')) AND (^IITR157 + Sum(ConditionalValue(NotInSet(^IITR482, '0,8'), ^IITR487, 0))) < 0 AND (ConditionalValue((^IITR490 + ^IITR491) > 3772, (^IITR490 + ^IITR491), 0) + ^IITR204 + ^IITR205 + ^IITR201 + Sum(^IITR362) - (^IITR153 - ^IITR154) - (^IITR155 - ^IITR156)) >= 250000
            
            Data Elements:
            
            RP.{LOSSSeqNum}:^IITR487 = tns:Tax.Losses.Net.Amount
            
            RP:^IITR19 = tns:Residency.TaxPurposesPersonStatus.Indicator
            
            RP.{LOSSSeqNum}:^IITR482 = tns:IncomeTax.LossesType.Code
            
            RP:^IITR157 = tns:Income.Taxable.Amount
            
            RP.Eligible:^IITR490 = tns:Income.FringeBenefitsReportable.Amount
            
            RP.Ineligible:^IITR491 = tns:Income.FringeBenefitsReportable.Amount
            
            RP:^IITR204 = tns:Tax.Losses.FinancialInvestmentNet.Amount
            
            RP:^IITR205 = tns:Tax.Losses.RentalPropertyNet.Amount
            
            RP:^IITR201 = tns:SuperannuationContribution.EmployerReportable.Amount
            
            RP.{SCSeqNum}:^IITR362 = tns:IncomeTax.Deduction.SuperannuationContribution.Amount
            
            RP:^IITR153 = tns:IncomeTax.LossesBroughtForwardPrimaryProduction.Amount
            
            RP:^IITR154 = tns:IncomeTax.LossesClaimedPrimaryProduction.Amount
            
            RP:^IITR155 = tns:IncomeTax.LossesBroughtForwardNonPrimaryProduction.Amount
            
            RP:^IITR156 = tns:IncomeTax.LossesClaimedNonPrimaryProduction.Amount
            */
            List<IITR2018.RPLOSSSeqNumContext> lossContextsIITR482_100032 = (report.RPLOSSSeqNumContextCollection != null ? report.RPLOSSSeqNumContextCollection.FindAll(f => !IsMatch(f.IITR482, "^(0|8)$")) : new List<IITR2018.RPLOSSSeqNumContext>());

            bool assertion = (report.IITR19 == true && IsMatch(rplossSeqNumContext.IITR482, "^([1-4])$") && (report.IITR157.GetValueOrDefault() + (lossContextsIITR482_100032.Count > 0 ? lossContextsIITR482_100032.Sum(f => f.IITR487.GetValueOrDefault()) : 0)) < 0 && ((report.IITR490.GetValueOrDefault() + report.IITR491.GetValueOrDefault() > 3772 ? report.IITR490.GetValueOrDefault() + report.IITR491.GetValueOrDefault() : 0) + report.IITR204.GetValueOrDefault() + report.IITR205.GetValueOrDefault() + report.IITR201.GetValueOrDefault() + (report.RPSCSeqNumContextCollection != null ? report.RPSCSeqNumContextCollection.Sum(f => f.IITR362.GetValueOrDefault()) : 0) - (report.IITR153.GetValueOrDefault() - report.IITR154.GetValueOrDefault()) - (report.IITR155.GetValueOrDefault() - report.IITR156.GetValueOrDefault())) >= 250000);

            return assertion;
        }

        #endregion // VR.ATO.IITR.100032

        #region VR.ATO.IITR.100033

        protected bool VRATOIITR100033(IITR2018 report, IITR2018.RPLOSSSeqNumContext rplossSeqNumContext)
        {
            /*  VR.ATO.IITR.100033
            For a non-resident taxpayer where the sum of taxable income or loss and the net loss amounts exceeds or equal to zero, the reported income for the purposes of the non-commercial losses income test is equal to $250,000 or more. Ensure loss codes used and/or amounts reported at the relevant labels are correct. A copy of the formula used to calculate income for this test can be found at www.ato.gov.au/ncl
    
            Legacy Rule Format:
            IF [iitr19] = FALSE AND ANY OCCURRENCE OF([iitr482]) = SET(1,2,3,4) AND ([iitr157] + SUM(ConditionalValue([iitr482] <> SET(0,8), [iitr487], 0)) + [iitr146] + [iitr147] - [iitr115] - [iitr118] - [iitr119] - [iitr120] - [iitr328]) >= 0 AND ([iitr157] + SUM(ConditionalValue([iitr482] <> SET(0,8), [iitr487], 0)) + ConditionalValue(([iitr490] + [iitr491]) > 3772, ([iitr490] + [iitr491]), 0) + [iitr204] + [iitr205] + [iitr201] + SUM([iitr362]) + [iitr146] + [iitr147] - [iitr115] - [iitr118] - [iitr119] - [iitr120] - [iitr328] - ([iitr153] - [iitr154]) - ([iitr155] - [iitr156])) >= 250000
                RETURN VALIDATION MESSAGE
            ENDIF
        
            Technical Business Rule Format:
            ^IITR19 = FALSE AND AnyOccurrence(^IITR482, InSet(^IITR482, '1,2,3,4')) AND (^IITR157 + Sum(ConditionalValue(NotInSet(^IITR482, '0,8'), ^IITR487, 0)) + ^IITR146 + ^IITR147 - ^IITR115 - ^IITR118 - ^IITR119 - ^IITR120 - ^IITR328) >= 0 AND (^IITR157 + Sum(ConditionalValue(NotInSet(^IITR482, '0,8'), ^IITR487, 0)) + ConditionalValue((^IITR490 + ^IITR491) > 3772, (^IITR490 + ^IITR491), 0) + ^IITR204 + ^IITR205 + ^IITR201 + Sum(^IITR362) + ^IITR146 + ^IITR147 - ^IITR115 - ^IITR118 - ^IITR119 - ^IITR120 - ^IITR328 - (^IITR153 - ^IITR154) - (^IITR155 - ^IITR156)) >= 250000
            
            Data Elements:
            
            RP.{LOSSSeqNum}:^IITR487 = tns:Tax.Losses.Net.Amount
            
            RP:^IITR19 = tns:Residency.TaxPurposesPersonStatus.Indicator
            
            RP.{LOSSSeqNum}:^IITR482 = tns:IncomeTax.LossesType.Code
            
            RP:^IITR157 = tns:Income.Taxable.Amount
            
            RP:^IITR146 = tns:IncomeTax.Deduction.Interest.Amount
            
            RP:^IITR147 = tns:IncomeTax.Deduction.Dividend.Amount
            
            RP:^IITR115 = tns:Income.Interest.Gross.Amount
            
            RP:^IITR118 = tns:Income.DividendsUnfranked.Amount
            
            RP:^IITR119 = tns:Income.DividendsFranked.Amount
            
            RP:^IITR120 = tns:IncomeTax.FrankingCredits.Amount
            
            RP:^IITR328 = tns:IncomeTax.FrankingCreditsNewZealand.Amount
            
            RP.Eligible:^IITR490 = tns:Income.FringeBenefitsReportable.Amount
            
            RP.Ineligible:^IITR491 = tns:Income.FringeBenefitsReportable.Amount
            
            RP:^IITR204 = tns:Tax.Losses.FinancialInvestmentNet.Amount
            
            RP:^IITR205 = tns:Tax.Losses.RentalPropertyNet.Amount
            
            RP:^IITR201 = tns:SuperannuationContribution.EmployerReportable.Amount
            
            RP.{SCSeqNum}:^IITR362 = tns:IncomeTax.Deduction.SuperannuationContribution.Amount
            
            RP:^IITR153 = tns:IncomeTax.LossesBroughtForwardPrimaryProduction.Amount
            
            RP:^IITR154 = tns:IncomeTax.LossesClaimedPrimaryProduction.Amount
            
            RP:^IITR155 = tns:IncomeTax.LossesBroughtForwardNonPrimaryProduction.Amount
            
            RP:^IITR156 = tns:IncomeTax.LossesClaimedNonPrimaryProduction.Amount
            */
            List<IITR2018.RPLOSSSeqNumContext> lossContextsIITR482_100033 = (report.RPLOSSSeqNumContextCollection != null ? report.RPLOSSSeqNumContextCollection.FindAll(f => !IsMatch(f.IITR482, "^(0|8)$")) : new List<IITR2018.RPLOSSSeqNumContext>());

            bool assertion = (report.IITR19 == false && IsMatch(rplossSeqNumContext.IITR482, "^([1-4])$") && (report.IITR157.GetValueOrDefault() + (lossContextsIITR482_100033.Count > 0 ? lossContextsIITR482_100033.Sum(f => f.IITR487.GetValueOrDefault()) : 0) + report.IITR146.GetValueOrDefault() + report.IITR147.GetValueOrDefault() - report.IITR115.GetValueOrDefault() - report.IITR118.GetValueOrDefault() - report.IITR119.GetValueOrDefault() - report.IITR120.GetValueOrDefault() - report.IITR328.GetValueOrDefault()) >= 0 && (report.IITR157.GetValueOrDefault() + (lossContextsIITR482_100033.Count > 0 ? lossContextsIITR482_100033.Sum(f => f.IITR487.GetValueOrDefault()) : 0) + (report.IITR490.GetValueOrDefault() + report.IITR491.GetValueOrDefault() > 3772 ? report.IITR490.GetValueOrDefault() + report.IITR491.GetValueOrDefault() : 0) + report.IITR204.GetValueOrDefault() + report.IITR205.GetValueOrDefault() + report.IITR201.GetValueOrDefault() + (report.RPSCSeqNumContextCollection != null ? report.RPSCSeqNumContextCollection.Sum(f => f.IITR362.GetValueOrDefault()) : 0) + report.IITR146.GetValueOrDefault() + report.IITR147.GetValueOrDefault() - report.IITR115.GetValueOrDefault() - report.IITR118.GetValueOrDefault() - report.IITR119.GetValueOrDefault() - report.IITR120.GetValueOrDefault() - report.IITR328.GetValueOrDefault() - (report.IITR153.GetValueOrDefault() - report.IITR154.GetValueOrDefault()) - (report.IITR155.GetValueOrDefault() - report.IITR156.GetValueOrDefault())) >= 250000);

            return assertion;
        }

        #endregion // VR.ATO.IITR.100033

        #region VR.ATO.IITR.100034

        protected bool VRATOIITR100034(IITR2018 report, IITR2018.RPLOSSSeqNumContext rplossSeqNumContext)
        {
            /*  VR.ATO.IITR.100034
            For a non-resident taxpayer where the sum of taxable income or loss and the net loss amounts is less than zero, the reported income for the purposes of the non-commercial losses income test is equal to $250,000 or more. Ensure loss codes used and/or amounts reported at the relevant labels are correct. A copy of the formula used to calculate income for this test can be found at www.ato.gov.au/ncl
    
            Legacy Rule Format:
            IF [iitr19] = FALSE AND ANY OCCURRENCE OF([iitr482]) = SET(1,2,3,4) AND ([iitr157] + SUM(ConditionalValue([iitr482] <> SET(0,8), [iitr487], 0)) + [iitr146] + [iitr147] - [iitr115] - [iitr118] - [iitr119] - [iitr120] - [iitr328]) < 0 AND (ConditionalValue(([iitr490] + [iitr491]) > 3772, ([iitr490] + [iitr491]), 0) + [iitr204] + [iitr205] + [iitr201] + SUM([iitr362]) - ([iitr153] - [iitr154]) - ([iitr155] - [iitr156])) >= 250000
                RETURN VALIDATION MESSAGE
            ENDIF
        
            Technical Business Rule Format:
            ^IITR19 = FALSE AND AnyOccurrence(^IITR482, InSet(^IITR482, '1,2,3,4')) AND (^IITR157 + Sum(ConditionalValue(NotInSet(^IITR482, '0,8'), ^IITR487, 0)) + ^IITR146 + ^IITR147 - ^IITR115 - ^IITR118 - ^IITR119 - ^IITR120 - ^IITR328) < 0 AND (ConditionalValue((^IITR490 + ^IITR491) > 3772, (^IITR490 + ^IITR491), 0) + ^IITR204 + ^IITR205 + ^IITR201 + Sum(^IITR362) - (^IITR153 - ^IITR154) - (^IITR155 - ^IITR156)) >= 250000
            
            Data Elements:
            
            RP.{LOSSSeqNum}:^IITR487 = tns:Tax.Losses.Net.Amount
            
            RP:^IITR19 = tns:Residency.TaxPurposesPersonStatus.Indicator
            
            RP.{LOSSSeqNum}:^IITR482 = tns:IncomeTax.LossesType.Code
            
            RP:^IITR157 = tns:Income.Taxable.Amount
            
            RP:^IITR146 = tns:IncomeTax.Deduction.Interest.Amount
            
            RP:^IITR147 = tns:IncomeTax.Deduction.Dividend.Amount
            
            RP:^IITR115 = tns:Income.Interest.Gross.Amount
            
            RP:^IITR118 = tns:Income.DividendsUnfranked.Amount
            
            RP:^IITR119 = tns:Income.DividendsFranked.Amount
            
            RP:^IITR120 = tns:IncomeTax.FrankingCredits.Amount
            
            RP:^IITR328 = tns:IncomeTax.FrankingCreditsNewZealand.Amount
            
            RP.Eligible:^IITR490 = tns:Income.FringeBenefitsReportable.Amount
            
            RP.Ineligible:^IITR491 = tns:Income.FringeBenefitsReportable.Amount
            
            RP:^IITR204 = tns:Tax.Losses.FinancialInvestmentNet.Amount
            
            RP:^IITR205 = tns:Tax.Losses.RentalPropertyNet.Amount
            
            RP:^IITR201 = tns:SuperannuationContribution.EmployerReportable.Amount
            
            RP.{SCSeqNum}:^IITR362 = tns:IncomeTax.Deduction.SuperannuationContribution.Amount
            
            RP:^IITR153 = tns:IncomeTax.LossesBroughtForwardPrimaryProduction.Amount
            
            RP:^IITR154 = tns:IncomeTax.LossesClaimedPrimaryProduction.Amount
            
            RP:^IITR155 = tns:IncomeTax.LossesBroughtForwardNonPrimaryProduction.Amount
            
            RP:^IITR156 = tns:IncomeTax.LossesClaimedNonPrimaryProduction.Amount
            */
            List<IITR2018.RPLOSSSeqNumContext> lossContextsIITR482_100033 = (report.RPLOSSSeqNumContextCollection != null ? report.RPLOSSSeqNumContextCollection.FindAll(f => !IsMatch(f.IITR482, "^(0|8)$")) : new List<IITR2018.RPLOSSSeqNumContext>());

            bool assertion = (report.IITR19 == false && IsMatch(rplossSeqNumContext.IITR482, "^([1-4])$") && (report.IITR157.GetValueOrDefault() + (lossContextsIITR482_100033.Count > 0 ? lossContextsIITR482_100033.Sum(f => f.IITR487.GetValueOrDefault()) : 0) + report.IITR146.GetValueOrDefault() + report.IITR147.GetValueOrDefault() - report.IITR115.GetValueOrDefault() - report.IITR118.GetValueOrDefault() - report.IITR119.GetValueOrDefault() - report.IITR120.GetValueOrDefault() - report.IITR328.GetValueOrDefault()) < 0 && ((report.IITR490.GetValueOrDefault() + report.IITR491.GetValueOrDefault() > 3772 ? report.IITR490.GetValueOrDefault() + report.IITR491.GetValueOrDefault() : 0) + report.IITR204.GetValueOrDefault() + report.IITR205.GetValueOrDefault() + report.IITR201.GetValueOrDefault() + (report.RPSCSeqNumContextCollection != null ? report.RPSCSeqNumContextCollection.Sum(f => f.IITR362.GetValueOrDefault()) : 0) - (report.IITR153.GetValueOrDefault() - report.IITR154.GetValueOrDefault()) - (report.IITR155.GetValueOrDefault() - report.IITR156.GetValueOrDefault())) >= 250000);

            return assertion;
        }

        #endregion // VR.ATO.IITR.100034
    }
}